package de.thefeiter.liedgutverzeichnis

import android.os.Bundle
import android.view.Menu
import android.view.MenuItem
import androidx.appcompat.app.AppCompatActivity
import androidx.appcompat.widget.Toolbar
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import de.thefeiter.liedgutverzeichnis.MainActivity.Companion.getAppDb
import de.thefeiter.liedgutverzeichnis.helpers.ConfigFilterHelper
import de.thefeiter.liedgutverzeichnis.objects.Category
import de.thefeiter.liedgutverzeichnis.objects.Song
import de.thefeiter.liedgutverzeichnis.ui.main.AdapterSongList

const val SORT_BY_TITLE = "de.thefeiter.liedgutverzeichnis.SongChooser.SORT_BY_TITLE"
const val SORT_BY_PAGE = "de.thefeiter.liedgutverzeichnis.SongChooser.SORT_BY_PAGE"
const val SORT_BY_BEGINNING =
    "de.thefeiter.liedgutverzeichnis.SongChooser.SORT_BY_BEGINNING"
const val NO_SORT = "de.thefeiter.liedgutverzeichnis.SongChooser.NO_SORT"
const val BOOK_ID = "de.thefeiter.liedgutverzeichnis.SongChooser.BOOK_ID"
const val BY_RANDOM = "de.thefeiter.liedgutverzeichnis.SongChooser.BY_RANDOM"
const val CATEGORY_ID = "de.thefeiter.liedgutverzeichnis.SongChooser.CATEGORY_ID"
const val SONG_LIST = "de.thefeiter.liedgutverzeichnis.SongChooser.SONG_LIST"

class SongChooser : AppCompatActivity() {
    lateinit var recyclerView: RecyclerView
    var songs: List<Song> = ArrayList()
    var appDb = getAppDb()
    var sortBy = SORT_BY_TITLE
    var filterSongs = true
    var pageSortEnabled = false
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_song_chooser)
        recyclerView = findViewById(R.id.songChooseRecyclerView)

        //adding the back button functionality
        // songChooseToolbar is defined in the layout file
        val myChildToolbar = findViewById<Toolbar>(R.id.songChooseToolbar)
        setSupportActionBar(myChildToolbar)

        // Get a support ActionBar corresponding to this toolbar and enable the Up button
        supportActionBar?.setDisplayHomeAsUpEnabled(true)

        // Get the Intent that started this activity and extract the string
        val intent = intent
        if (intent.hasExtra(SONG_LIST)) {
            val ids = intent.getSerializableExtra(SONG_LIST) as IntArray?
            songs = appDb.songDao().loadAllByIds(ids)
        } else if (intent.hasExtra(BOOK_ID)) {
            val bookId = intent.getSerializableExtra(BOOK_ID) as Int
            val book = appDb.bookDao().findById(bookId)
            supportActionBar?.title = book.name
            songs = appDb.songDao().loadAllByBookId(book.id)
            sortBy = SORT_BY_PAGE
            filterSongs = false
            pageSortEnabled = true
        } else if (intent.hasExtra(CATEGORY_ID)) {
            val categoryId = intent.getSerializableExtra(CATEGORY_ID) as Int
            val category = appDb.categoryDao().findById(categoryId)
            if (category.id == Category.UNCATEGORIZED) {
                songs = appDb.songDao().loadUncategorized()
            } else {
                songs = appDb.songDao().loadAllByCategoryFilteredByConfig(categoryId)
                supportActionBar!!.title = category.name
            }
        } else {
            finish()
        }

        //filter songs
        if (filterSongs) {
            songs = ConfigFilterHelper().filterSongsByConfig(appDb, songs)
        }


        // show search results in recycler view
        recyclerView.setHasFixedSize(true)
        recyclerView.layoutManager = LinearLayoutManager(this)
        recyclerView.adapter = AdapterSongList(songs, sortBy)
    }

    override fun onPrepareOptionsMenu(menu: Menu?): Boolean {
        super.onPrepareOptionsMenu(menu)

        //add topflop button to menu if superuser
        menu?.findItem(R.id.sortBy_page)?.isVisible = pageSortEnabled
        return true
    }

    override fun onCreateOptionsMenu(menu: Menu): Boolean {
        val inflater = menuInflater
        inflater.inflate(R.menu.menu_sort, menu)
        return true
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        val adapter = recyclerView.adapter as AdapterSongList?
        val itemId = item.itemId
        if (itemId == android.R.id.home) { // User chose the Back Button, show the app settings UI...
            finish()
            return true
        } else if (itemId == R.id.sortBy_name && adapter != null) {
            adapter.sort(SORT_BY_TITLE)
            return true
        } else if (itemId == R.id.sortBy_page && adapter != null) {
            adapter.sort(SORT_BY_PAGE)
            return true
        } else if (itemId == R.id.sortBy_beginning && adapter != null) {
            adapter.sort(SORT_BY_BEGINNING)
            return true
        } // If we got here, the user's action was not recognized.
        // Invoke the superclass to handle it.
        return super.onOptionsItemSelected(item)
    }

}