package de.thefeiter.liedgutverzeichnis.api

import android.content.Context
import android.content.SharedPreferences
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import androidx.fragment.app.FragmentManager
import com.android.volley.Request
import com.android.volley.Response
import com.android.volley.VolleyError
import com.android.volley.toolbox.JsonObjectRequest
import de.thefeiter.liedgutverzeichnis.R
import de.thefeiter.liedgutverzeichnis.api.updater.UpdaterAll
import de.thefeiter.liedgutverzeichnis.helpers.SyncHelper
import de.thefeiter.liedgutverzeichnis.objects.User
import de.thefeiter.liedgutverzeichnis.roomdb.AppDb
import de.thefeiter.liedgutverzeichnis.ui.dialogs.UserSetupDialog
import de.thefeiter.liedgutverzeichnis.ui.settings.PREFERENCE_NAME
import de.thefeiter.liedgutverzeichnis.ui.settings.SHARED_PREFS_USER_API_TOKEN
import de.thefeiter.liedgutverzeichnis.ui.settings.SHARED_PREFS_USER_SUPERUSER
import de.thefeiter.liedgutverzeichnis.ui.settings.SHARED_PREFS_USER_TOPFLOP_ALLOWED
import org.json.JSONException
import org.json.JSONObject


class UserManager(
    private val context: Context,
    private val db: AppDb,
    private val fragmentManager: FragmentManager,
    private val syncHelper: SyncHelper
) {
    fun testToken(testToken: String) {
        ApiTest(context).test(db, { getUserdata(testToken) }, testToken)
    }

    private fun getUserdata(testToken: String) {
        val whoamiRequest: JsonObjectRequest = object : JsonObjectRequest(
            Method.GET,
            context.getString(R.string.api_base_url) + context.getString(R.string.endpoint_whoami),
            null,
            Response.Listener { result: JSONObject -> saveUserToDb(result, testToken) },
            Response.ErrorListener { error: VolleyError -> onTokenTestError(error) }) {
            override fun getHeaders(): Map<String, String> {
                val headers = HashMap<String, String>()
                headers[context.getString(R.string.api_headers_token)] = testToken
                return headers
            }
        }


        // Access the RequestQueue through your singleton class.
        MySingleton.getInstance(context).addToRequestQueue(whoamiRequest)
    }

    private fun onTokenTestError(error: VolleyError) {
        error.printStackTrace()
        reshowDialog(context.getString(R.string.token_invalid))
    }

    fun saveRegisteredUserToDb(result: JSONObject) {
        //token seems to be valid
        val apiToken: String
        try {
            apiToken = result.getString(context.getString(R.string.json_key_apiToken))
        } catch (e: JSONException) {
            e.printStackTrace()
            reshowDialog(context.getString(R.string.error_from_api))
            return
        }

        saveUserToDb(result, apiToken)
    }

    fun saveUserToDb(result: JSONObject, apiToken: String) {
        //token seems to be valid
        //val apiToken: String
        val superuser: Boolean
        val topflop: Boolean
        try {
            //apiToken = result.getString(context.getString(R.string.json_key_apiToken))
            superuser = result.getBoolean(context.getString(R.string.json_key_superuser))
            topflop = result.getBoolean(context.getString(R.string.json_key_topflop_allowed))
        } catch (e: JSONException) {
            e.printStackTrace()
            reshowDialog(context.getString(R.string.error_from_api))
            return
        }


        val sharedPreferences: SharedPreferences =
            context.getSharedPreferences(PREFERENCE_NAME, AppCompatActivity.MODE_PRIVATE)

        sharedPreferences.edit()
            .putString(SHARED_PREFS_USER_API_TOKEN, apiToken)
            .apply()

        sharedPreferences.edit()
            .putBoolean(SHARED_PREFS_USER_SUPERUSER, superuser)
            .apply()

        sharedPreferences.edit()
            .putBoolean(SHARED_PREFS_USER_TOPFLOP_ALLOWED, topflop)
            .apply()

        val user = User()
        user.apiToken = apiToken
        user.superuser = superuser
        user.allowTopflop = topflop

        //update all possible tables
        UpdaterAll(context, syncHelper, null, {}) {}.update(db)
    }

    fun registerUser(username: String) {
        if (ApiTest(context).networkUnreachable()) {
            reshowDialog(context.getString(R.string.no_connection))
            return
        }
        val url =
            context.getString(R.string.api_base_url) + context.getString(R.string.endpoint_register)
                .format(username)


        val jsonObjectRequest = JsonObjectRequest(
            Request.Method.POST,
            url,
            null,
            { result: JSONObject -> saveRegisteredUserToDb(result) },
            { error: VolleyError ->
                onRegistrationError(
                    error
                )
            })


        // Access the RequestQueue through your singleton class.
        MySingleton.getInstance(context).addToRequestQueue(jsonObjectRequest)
    }

    private fun onRegistrationError(error: VolleyError) {
        error.printStackTrace()
        reshowDialog(context.getString(R.string.registration_failed))
    }

    private fun reshowDialog(msg: String?) {
        Toast.makeText(context, msg, Toast.LENGTH_LONG).show()
        val frag = UserSetupDialog(db, context, fragmentManager, syncHelper)
        frag.show()
    }
}