package de.thefeiter.liedgutverzeichnis.api.loader

import android.content.Context
import androidx.fragment.app.Fragment
import androidx.lifecycle.lifecycleScope
import com.android.volley.Response
import com.android.volley.VolleyError
import com.android.volley.toolbox.JsonArrayRequest
import de.thefeiter.liedgutverzeichnis.R
import de.thefeiter.liedgutverzeichnis.api.MySingleton
import de.thefeiter.liedgutverzeichnis.objects.User
import de.thefeiter.liedgutverzeichnis.roomdb.AppDb
import kotlinx.coroutines.launch
import org.json.JSONArray

open class LoaderArray(
    protected val ctx: Context,
    protected val db: AppDb,
    protected val onResponseCallback: () -> Unit,
    protected val onCompleteCallback: () -> Unit,
    private val url: String
) : Fragment() {


    @JvmField
    protected val user = User()

    protected fun handleError(error: VolleyError) {
        error.printStackTrace()
    }

    open suspend fun onResponse(response: JSONArray) {
        onResponseCallback.invoke()
        onCompleteCallback.invoke()
    }

    private fun getRequest(url: String): JsonArrayRequest {
        return object : JsonArrayRequest(
            Method.GET,
            url,
            null,
            Response.Listener { response: JSONArray -> lifecycleScope.launch { onResponse(response) } },
            Response.ErrorListener { error: VolleyError -> handleError(error) }) {
            /**
             * Passing some request headers
             */
            override fun getHeaders(): Map<String, String> {
                val headers = HashMap<String, String>()
                headers[ctx.getString(R.string.api_headers_token)] = user.apiToken
                return headers
            }
        }
    }

    fun load() {
        val jsonArrayRequest = getRequest(url)

        // Access the RequestQueue through your singleton class.
        MySingleton.getInstance(ctx).addToRequestQueue(jsonArrayRequest)
    }


}