package de.thefeiter.liedgutverzeichnis.api.loader

import android.content.Context
import android.widget.Toast
import androidx.fragment.app.Fragment
import androidx.lifecycle.lifecycleScope
import com.android.volley.Request.Method
import com.android.volley.Response
import com.android.volley.VolleyError
import com.android.volley.toolbox.JsonObjectRequest
import de.thefeiter.liedgutverzeichnis.R
import de.thefeiter.liedgutverzeichnis.api.MySingleton
import de.thefeiter.liedgutverzeichnis.objects.User
import de.thefeiter.liedgutverzeichnis.roomdb.AppDb
import kotlinx.coroutines.launch
import org.json.JSONObject

open class LoaderObject(
    ctx: Context,
    protected val db: AppDb,
    protected val completeCallback: () -> Unit,
    private val url: String,
    private val requestMethod: Int = Method.GET
) : Fragment() {

    protected val ctx: Context = ctx.applicationContext

    protected val user = User()

    protected fun handleError(error: VolleyError) {
        error.printStackTrace()
    }

    private fun errorToast(msg: String) {
        Toast.makeText(ctx, msg, Toast.LENGTH_SHORT).show()
    }

    open suspend fun onResponse(response: JSONObject) {
        completeCallback.invoke()
    }

    open fun getRequest(url: String): JsonObjectRequest {
        return object : JsonObjectRequest(
            requestMethod,
            url,
            null,
            Response.Listener { response: JSONObject -> lifecycleScope.launch { onResponse(response) } },
            Response.ErrorListener { error: VolleyError -> handleError(error) }) {
            /**
             * Passing some request headers
             */
            override fun getHeaders(): Map<String, String> {
                val headers = HashMap<String, String>()
                headers[ctx.getString(R.string.api_headers_token)] = user.apiToken
                return headers
            }
        }
    }

    fun load() {
        val jsonObjectRequest = getRequest(url)

        // Access the RequestQueue through your singleton class.
        MySingleton.getInstance(ctx).addToRequestQueue(jsonObjectRequest)
    }

}