package de.thefeiter.liedgutverzeichnis.api.loader

import android.content.Context
import android.content.Intent
import android.util.Log
import androidx.core.content.FileProvider
import com.android.volley.Response
import com.android.volley.VolleyError
import de.thefeiter.liedgutverzeichnis.R
import de.thefeiter.liedgutverzeichnis.api.ApiTest
import de.thefeiter.liedgutverzeichnis.api.MySingleton
import de.thefeiter.liedgutverzeichnis.objects.Song
import de.thefeiter.liedgutverzeichnis.roomdb.AppDb
import java.io.File
import java.io.FileOutputStream

class LoaderPdf(song: Song, db: AppDb, context: Context, private val onSuccess: () -> Unit) :
    LoaderFile(
        context,
        db,
        Runnable {},
        context.getString(R.string.api_base_url) + context.getString(R.string.endpoint_pdf_file)
            .format(song.pdfId)
    ) {
    private val filename: String = song.title + ".pdf"


    override val request: InputStreamVolleyRequest
        get() = object : InputStreamVolleyRequest(
            Method.GET,
            url,
            Response.Listener { response: ByteArray? -> onResponse(response) },
            Response.ErrorListener { error: VolleyError? ->
                onError(
                    error!!
                )
            },
            null
        ) {
            override fun getHeaders(): Map<String, String> {
                val headers = HashMap<String, String>()
                headers[context.getString(R.string.api_headers_token)] = user.apiToken
                return headers
            }
        }

    override fun onResponse(response: ByteArray?) {
        try {
            if (response != null) {
                val outputStream: FileOutputStream =
                    context.openFileOutput(filename, Context.MODE_PRIVATE)
                outputStream.write(response)
                outputStream.close()
                //Toast.makeText(context, "Download complete.", Toast.LENGTH_LONG).show()
                openPdfViewer()
            }
        } catch (e: Exception) {
            Log.d("KEY_ERROR", "UNABLE TO DOWNLOAD FILE")
            e.printStackTrace()
        }
    }

    fun loadPdf() {
        val file = File(context.filesDir, filename)
        if (file.exists()) {
            openPdfViewer()
        } else {
            //test if api is working
            ApiTest(context).test(db) {
                MySingleton.getInstance(context).addToRequestQueue(request)
            }
        }
    }

    private fun openPdfViewer() {
        // Get URI and MIME type of file
        onSuccess.invoke()
        val uri = FileProvider.getUriForFile(
            context,
            context.applicationContext.packageName + ".provider",
            context.getFileStreamPath(filename)
        )
        val mime = context.contentResolver.getType(uri)


        // Open file with user selected app
        val intent = Intent()
        intent.action = Intent.ACTION_VIEW
        intent.setDataAndType(uri, mime)
        intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION)
        context.startActivity(intent)
    }

}