package de.thefeiter.liedgutverzeichnis.api.syncer

import android.content.Context
import com.android.volley.Request
import de.thefeiter.liedgutverzeichnis.R
import de.thefeiter.liedgutverzeichnis.api.ApiTest
import de.thefeiter.liedgutverzeichnis.api.loader.LoaderArray
import de.thefeiter.liedgutverzeichnis.api.loader.LoaderObject
import de.thefeiter.liedgutverzeichnis.objects.BookSetting
import de.thefeiter.liedgutverzeichnis.roomdb.AppDb
import org.json.JSONArray
import org.json.JSONException

class SyncerBookSettings(
    private val context: Context,
    private val db: AppDb,
    private val onResponse: () -> Unit,
    private val onComplete: () -> Unit
) {
    fun sync() {
        val apiTester = ApiTest(context)
        apiTester.test(db) { testedSync() }
    }

    fun testedSync() {
        val remoteBookSettings: MutableList<BookSetting> = ArrayList()
        val bookSettingsLoader: LoaderArray = object : LoaderArray(
            context,
            db,
            onResponse,
            {},
            context.getString(R.string.api_base_url) + context.getString(R.string.endpoint_book_config)
        ) {
            override suspend fun onResponse(response: JSONArray) {
                onResponseCallback.invoke()
                remoteBookSettings.clear()
                try {
                    for (i in 0 until response.length()) {
                        val remoteBS = response.getJSONObject(i)
                        val bookSetting = BookSetting()
                        bookSetting.bookId =
                            remoteBS.getInt(ctx.getString(R.string.json_key_bookId))
                        bookSetting.active =
                            remoteBS.getBoolean(ctx.getString(R.string.json_key_active))
                        bookSetting.updated =
                            remoteBS.getDouble(ctx.getString(R.string.json_key_updated))
                        remoteBookSettings.add(bookSetting)
                    }
                } catch (e: JSONException) {
                    e.printStackTrace()
                }
                merge(remoteBookSettings, db.bookSettingDao().getAll())
            }
        }
        bookSettingsLoader.load()
    }

    private fun merge(
        remoteBookSettings: List<BookSetting>,
        localBookSettings: List<BookSetting>
    ) {
        val bookSettingsToPush: MutableList<BookSetting> = ArrayList()

        // sync timestamps on favorites existing on both sides
        for (localBookSetting in localBookSettings) {
            for (remoteBookSetting in remoteBookSettings) {
                if (localBookSetting.bookId == remoteBookSetting.bookId) {

                    //if remote is newer than local
                    if (remoteBookSetting.updated.toInt() > localBookSetting.updated.toInt()) {
                        localBookSetting.active = remoteBookSetting.active
                        localBookSetting.updated = remoteBookSetting.updated
                        db.bookSettingDao().update(
                            remoteBookSetting.active,
                            remoteBookSetting.updated,
                            remoteBookSetting.bookId,
                        )
                    }

                    //if local is newer than remote
                    if (localBookSetting.updated.toInt() > remoteBookSetting.updated.toInt()) {
                        remoteBookSetting.active = localBookSetting.active
                        remoteBookSetting.updated = localBookSetting.updated
                        bookSettingsToPush.add(remoteBookSetting)
                    }
                }
            }
        }

        //add favorites on local which only exist remotely
        for (localBookSetting in localBookSettings) {
            var existsOnRemote = false
            for (remoteBookSetting in remoteBookSettings) {
                if (localBookSetting.bookId == remoteBookSetting.bookId) {
                    existsOnRemote = true
                    break
                }
            }
            if (!existsOnRemote) {
                bookSettingsToPush.add(localBookSetting)
            }
        }

        //add favorites on remote which only exist locally
        for (remoteBookSetting in remoteBookSettings) {
            var existsOnLocal = false
            for (localBookSetting in localBookSettings) {
                if (localBookSetting.bookId == remoteBookSetting.bookId) {
                    existsOnLocal = true
                    break
                }
            }
            if (!existsOnLocal) {
                //add to local favorites
                db.bookSettingDao().insert(remoteBookSetting)
            }
        }
        updateRemoteBooks(bookSettingsToPush)
    }

    private fun pushBookSettings(localBookSettings: MutableList<BookSetting>) {
        if (localBookSettings.size > 0) {
            val bookSettingPusher = LoaderObject(
                context,
                db,
                {
                    localBookSettings.removeAt(0)
                    pushBookSettings(localBookSettings)
                },
                context.getString(R.string.api_base_url) + context.getString(R.string.endpoint_book_config_single_put)
                    .format(
                        localBookSettings[0].bookId,
                        localBookSettings[0].active,
                        localBookSettings[0].updated.toInt()
                    ),
                Request.Method.PUT
            )
            bookSettingPusher.load()
        } else {
            onComplete.invoke()
        }

    }

    private fun updateRemoteBooks(bookSettingsToPush: MutableList<BookSetting>) {
        pushBookSettings(bookSettingsToPush)
    }


}