package de.thefeiter.liedgutverzeichnis.api.syncer

import android.content.Context
import androidx.lifecycle.lifecycleScope
import com.android.volley.Request
import de.thefeiter.liedgutverzeichnis.R
import de.thefeiter.liedgutverzeichnis.api.ApiTest
import de.thefeiter.liedgutverzeichnis.api.loader.LoaderArray
import de.thefeiter.liedgutverzeichnis.api.loader.LoaderObject
import de.thefeiter.liedgutverzeichnis.objects.Bookmark
import de.thefeiter.liedgutverzeichnis.objects.Song
import de.thefeiter.liedgutverzeichnis.roomdb.AppDb
import kotlinx.coroutines.launch
import org.json.JSONArray
import org.json.JSONException


class SyncerBookmarks(
    private val context: Context,
    private val db: AppDb,
    private val onResponse: () -> Unit,
    private val onComplete: () -> Unit
) {
    private val remoteBookmarks: MutableList<Bookmark> = ArrayList()
    private val bookmarksToPush: MutableList<Song> = ArrayList()
    fun sync() {
        val apiTester = ApiTest(context)
        apiTester.test(db) { testedSync() }
    }

    fun testedSync() {
        val bookmarkLoader: LoaderArray = object : LoaderArray(
            context,
            db,
            onResponse,
            {},
            context.getString(R.string.api_base_url) + context.getString(R.string.endpoint_bookmarks_get)
        ) {
            override suspend fun onResponse(response: JSONArray) {
                onResponseCallback.invoke()
                lifecycleScope.launch {
                    remoteBookmarks.clear()
                    try {
                        for (i in 0 until response.length()) {
                            val remoteBm = response.getJSONObject(i)
                            val bm = Bookmark()
                            bm.songId = remoteBm.getInt(ctx.getString(R.string.json_key_songId))
                            bm.active =
                                remoteBm.getBoolean(ctx.getString(R.string.json_key_active))
                            bm.updated =
                                remoteBm.getDouble(ctx.getString(R.string.json_key_updated))
                            remoteBookmarks.add(bm)
                        }
                    } catch (e: JSONException) {
                        e.printStackTrace()
                    }
                    merge()
                    onCompleteCallback.invoke()
                }

            }
        }
        bookmarkLoader.load()
    }

    private fun merge() {
        val localSongsEverBeenBookmark = db.songDao().getAllEverBeenBookmarked()

        // sync timestamps on bookmarks existing on both sides
        for (localSong in localSongsEverBeenBookmark) {
            for (remoteBookmark in remoteBookmarks) {
                if (localSong.id == remoteBookmark.songId) {

                    //if remote is newer than local
                    if (remoteBookmark.updated.toInt() > localSong.bookmarkUpdateTimestamp.toInt()) {

                        db.songDao().updateBookmarkState(
                            remoteBookmark.songId,
                            remoteBookmark.active,
                            remoteBookmark.updated

                        )
                    }

                    //if local is newer than remote
                    if (localSong.bookmarkUpdateTimestamp.toInt() > remoteBookmark.updated.toInt()) {
                        bookmarksToPush.add(localSong)
                    }
                }
            }
        }

        //add bookmarks to remote db which only exist locally
        for (localSong in localSongsEverBeenBookmark) {
            var existsOnRemote = false
            for (remoteBookmark in remoteBookmarks) {
                if (localSong.id == remoteBookmark.songId) {
                    existsOnRemote = true
                    break
                }
            }
            if (!existsOnRemote) {
                bookmarksToPush.add(localSong)
            }
        }

        //add bookmarks to local db which only exist remotely
        for (remoteBookmark in remoteBookmarks) {
            var existsOnLocal = false
            for (localSong in localSongsEverBeenBookmark) {
                if (localSong.id == remoteBookmark.songId) {
                    existsOnLocal = true
                    break
                }
            }
            if (!existsOnLocal) {
                //add to local bookmarks
                db.songDao().updateBookmarkState(
                    remoteBookmark.songId,
                    remoteBookmark.active,
                    remoteBookmark.updated
                )
            }
        }
        updateRemoteBookmarks()
    }

    private fun pushBookmarks(localSongs: MutableList<Song>) {
        if (localSongs.size > 0) {
            val bookmarkPusher = LoaderObject(
                context,
                db,
                {
                    localSongs.removeAt(0)
                    pushBookmarks(localSongs)
                },
                context.getString(R.string.api_base_url) + context.getString(R.string.endpoint_bookmark_put)
                    .format(
                        localSongs[0].id,
                        localSongs[0].bookmark,
                        localSongs[0].bookmarkUpdateTimestamp.toInt()
                    ),
                Request.Method.PUT
            )
            bookmarkPusher.load()
        } else {
            onComplete.invoke()
        }

    }

    private fun updateRemoteBookmarks() {
        pushBookmarks(bookmarksToPush)
    }
}