package de.thefeiter.liedgutverzeichnis.api.syncer

import android.content.Context
import androidx.lifecycle.lifecycleScope
import com.android.volley.Request
import de.thefeiter.liedgutverzeichnis.R
import de.thefeiter.liedgutverzeichnis.api.ApiTest
import de.thefeiter.liedgutverzeichnis.api.loader.LoaderArray
import de.thefeiter.liedgutverzeichnis.api.loader.LoaderObject
import de.thefeiter.liedgutverzeichnis.objects.Favorite
import de.thefeiter.liedgutverzeichnis.objects.Song
import de.thefeiter.liedgutverzeichnis.roomdb.AppDb
import kotlinx.coroutines.launch
import org.json.JSONArray
import org.json.JSONException

class SyncerFavorites(
    private val context: Context,
    private val db: AppDb,
    private val onResponse: () -> Unit,
    private val onComplete: () -> Unit
) {
    private val remoteFavorites: MutableList<Favorite> = ArrayList()
    private val favoritesToPush: MutableList<Song> = ArrayList()
    fun sync() {
        val apiTester = ApiTest(context)
        apiTester.test(db) { testedSync() }
    }

    fun testedSync() {
        val favoriteLoader: LoaderArray = object : LoaderArray(
            context,
            db,
            onResponse,
            {},
            context.getString(R.string.api_base_url) + context.getString(R.string.endpoint_favorites_get)
        ) {
            override suspend fun onResponse(response: JSONArray) {
                onResponseCallback.invoke()
                lifecycleScope.launch {
                    remoteFavorites.clear()
                    try {
                        for (i in 0 until response.length()) {
                            val remoteFav = response.getJSONObject(i)
                            val fav = Favorite()
                            fav.songId = remoteFav.getInt(ctx.getString(R.string.json_key_songId))
                            fav.active =
                                remoteFav.getBoolean(ctx.getString(R.string.json_key_active))
                            fav.updated =
                                remoteFav.getDouble(ctx.getString(R.string.json_key_updated))
                            remoteFavorites.add(fav)
                        }
                    } catch (e: JSONException) {
                        e.printStackTrace()
                    }
                    merge()
                }

            }
        }
        favoriteLoader.load()
    }

    private fun merge() {
        val localSongsEverBeenFavorite = db.songDao().getAllEverBeenFavorite()

        // sync timestamps on favorites existing on both sides
        for (localSong in localSongsEverBeenFavorite) {
            for (remoteFavorite in remoteFavorites) {
                if (localSong.id == remoteFavorite.songId) {

                    //if remote is newer than local
                    if (remoteFavorite.updated.toInt() > localSong.favoriteUpdateTimestamp.toInt()) {
                        //localSong.favorite = remoteFavorite.active
                        //localSong.favoriteUpdateTimestamp = remoteFavorite.updated
                        db.songDao().updateFavoriteState(
                            remoteFavorite.songId,
                            remoteFavorite.active,
                            remoteFavorite.updated

                        )
                    }

                    //if local is newer than remote
                    if (localSong.favoriteUpdateTimestamp.toInt() > remoteFavorite.updated.toInt()) {
                        //remoteFavorite.active = localSong.favorite
                        //remoteFavorite.updated = localSong.favoriteUpdateTimestamp
                        favoritesToPush.add(localSong)
                    }
                }
            }
        }

        //add favorites to remote db which only exist locally
        for (localSong in localSongsEverBeenFavorite) {
            var existsOnRemote = false
            for (remoteFavorite in remoteFavorites) {
                if (localSong.id == remoteFavorite.songId) {
                    existsOnRemote = true
                    break
                }
            }
            if (!existsOnRemote) {
                favoritesToPush.add(localSong)
            }
        }

        //add favorites to local db which only exist remotely
        for (remoteFavorite in remoteFavorites) {
            var existsOnLocal = false
            for (localSong in localSongsEverBeenFavorite) {
                if (localSong.id == remoteFavorite.songId) {
                    existsOnLocal = true
                    break
                }
            }
            if (!existsOnLocal) {
                //add to local favorites
                //db.favoriteDao().insert(remoteFavorite)
                db.songDao().updateFavoriteState(
                    remoteFavorite.songId,
                    remoteFavorite.active,
                    remoteFavorite.updated
                )
            }
        }
        updateRemoteFavorites()
    }

    private fun pushFavorites(localFavorites: MutableList<Song>) {
        if (localFavorites.size > 0) {
            val favoritesPusher = LoaderObject(
                context,
                db,
                {
                    localFavorites.removeAt(0)
                    pushFavorites(localFavorites)
                },
                context.getString(R.string.api_base_url) + context.getString(R.string.endpoint_favorite_put)
                    .format(
                        localFavorites[0].id,
                        localFavorites[0].favorite,
                        localFavorites[0].favoriteUpdateTimestamp.toInt()
                    ),
                Request.Method.PUT
            )
            favoritesPusher.load()
        } else {
            onComplete.invoke()
        }

    }

    private fun updateRemoteFavorites() {
        pushFavorites(favoritesToPush)
    }

}