package de.thefeiter.liedgutverzeichnis.api.updater

import android.content.Context
import android.content.Context.MODE_PRIVATE
import android.widget.ProgressBar
import de.thefeiter.liedgutverzeichnis.MainActivity.Companion.timestamp
import de.thefeiter.liedgutverzeichnis.api.ApiTest
import de.thefeiter.liedgutverzeichnis.api.syncer.SyncerBookSettings
import de.thefeiter.liedgutverzeichnis.api.syncer.SyncerBookmarks
import de.thefeiter.liedgutverzeichnis.api.syncer.SyncerFavorites
import de.thefeiter.liedgutverzeichnis.helpers.SyncHelper
import de.thefeiter.liedgutverzeichnis.roomdb.AppDb
import de.thefeiter.liedgutverzeichnis.ui.settings.PREFERENCE_NAME
import de.thefeiter.liedgutverzeichnis.ui.settings.SHARED_PREFS_LAST_DB_SYNC

const val MAX_PROGRESS = 10
const val AUTO_SYNC_INTERVAL = 3600 * 24

class UpdaterAll(
    private val context: Context,
    private val syncHelper: SyncHelper,
    private val progressBar: ProgressBar?,
    private val onBegin: () -> Unit,
    private val onComplete: () -> Unit
) {

    fun update(appDb: AppDb) {
        //test if api is working
        ApiTest(context).test(appDb) {
            updateAll(appDb)
        }
    }

    private fun increaseProgress() {
        progressBar?.let {
            it.progress++
            if (it.progress >= MAX_PROGRESS) {
                it.progress = 0
                onComplete.invoke()
                syncHelper.setSyncState(false)
                context.getSharedPreferences(PREFERENCE_NAME, MODE_PRIVATE).edit()
                    .putInt(SHARED_PREFS_LAST_DB_SYNC, timestamp.toInt()).apply()
            }
        }
    }

    private fun updateAll(appDb: AppDb) {
        progressBar?.max = MAX_PROGRESS
        progressBar?.progress = 0
        onBegin.invoke()

        syncHelper.setSyncState(true)

        val categoryUpdater = UpdaterCategory(context.applicationContext, appDb, {}) {
            //hide loading animation
            increaseProgress()
        }

        val favoritesSyncer =
            SyncerFavorites(context.applicationContext, appDb, { categoryUpdater.load() }) {
                increaseProgress()
            }

        val bookmarksSyncer =
            SyncerBookmarks(context.applicationContext, appDb, { favoritesSyncer.testedSync() }) {
                increaseProgress()
            }

        val categoryRelationUpdater = UpdaterCategoryRelations(
            context.applicationContext,
            appDb,
            { bookmarksSyncer.testedSync() }) {
            increaseProgress()
        }

        val bookUpdater =
            UpdaterBook(context.applicationContext, appDb, { categoryRelationUpdater.load() }) {
                increaseProgress()
            }

        val bookRelationsUpdater =
            UpdaterBookRelations(context.applicationContext, appDb, { bookUpdater.load() }) {
                increaseProgress()
            }

        val songUpdater =
            UpdaterSong(context.applicationContext, appDb, { bookRelationsUpdater.load() }) {
                increaseProgress()
            }

        val bookSettingsSyncer =
            SyncerBookSettings(context.applicationContext, appDb, { songUpdater.load() }) {
                increaseProgress()
            }

        val songSuggestionsUpdater = UpdaterSongSuggestions(
            context.applicationContext,
            appDb,
            { bookSettingsSyncer.testedSync() }) {
            increaseProgress()
        }

        UpdaterUserPermissions(context.applicationContext, appDb) {
            increaseProgress()
            songSuggestionsUpdater.load()
        }.load()
    }

}