package de.thefeiter.liedgutverzeichnis.api.updater

import android.content.Context
import de.thefeiter.liedgutverzeichnis.R
import de.thefeiter.liedgutverzeichnis.api.loader.LoaderArray
import de.thefeiter.liedgutverzeichnis.exceptions.DatabaseUpdateException
import de.thefeiter.liedgutverzeichnis.objects.Book
import de.thefeiter.liedgutverzeichnis.roomdb.AppDb
import kotlinx.coroutines.DelicateCoroutinesApi
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.launch
import org.json.JSONArray
import org.json.JSONException

class UpdaterBook(
    ctx: Context,
    db: AppDb,
    responseCallback: () -> Unit,
    completeCallback: () -> Unit
) : LoaderArray(
    ctx,
    db,
    responseCallback,
    completeCallback,
    ctx.getString(R.string.api_base_url) + ctx.getString(R.string.endpoint_book)
) {
    @OptIn(DelicateCoroutinesApi::class)
    override suspend fun onResponse(response: JSONArray) {
        onResponseCallback.invoke()
        GlobalScope.launch {
            val bookDao = db.bookDao()
            bookDao.deleteAll()
            for (i in 0 until response.length()) {
                try {
                    val booksJSONObject = response.getJSONObject(i)
                    val book = Book()
                    book.id = booksJSONObject.getInt(ctx.getString(R.string.json_key_bookId))
                    book.name = booksJSONObject.getString(ctx.getString(R.string.json_key_bookName))
                    bookDao.insert(book)
                } catch (e: JSONException) {
                    DatabaseUpdateException(e.message).notifyUser(ctx)
                }
            }
            onCompleteCallback.invoke()
        }

    }
}