package de.thefeiter.liedgutverzeichnis.api.updater

import android.content.Context
import de.thefeiter.liedgutverzeichnis.R
import de.thefeiter.liedgutverzeichnis.api.loader.LoaderArray
import de.thefeiter.liedgutverzeichnis.exceptions.DatabaseUpdateException
import de.thefeiter.liedgutverzeichnis.objects.BookRelation
import de.thefeiter.liedgutverzeichnis.roomdb.AppDb
import kotlinx.coroutines.DelicateCoroutinesApi
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.launch
import org.json.JSONArray
import org.json.JSONException

class UpdaterBookRelations(
    ctx: Context,
    db: AppDb,
    responseCallback: () -> Unit,
    completeCallback: () -> Unit
) :
    LoaderArray(
        ctx,
        db,
        responseCallback,
        completeCallback,
        ctx.getString(R.string.api_base_url) + ctx.getString(R.string.endpoint_book_relation)
    ) {
    @OptIn(DelicateCoroutinesApi::class)
    override suspend fun onResponse(response: JSONArray) {
        onResponseCallback.invoke()
        GlobalScope.launch {
            val bookRelationDao = db.bookRelationDao()
            bookRelationDao.deleteAll()
            for (i in 0 until response.length()) {
                try {
                    val bookRelationsJSONObject = response.getJSONObject(i)
                    val bookRelation = BookRelation()
                    bookRelation.bookId =
                        bookRelationsJSONObject.getInt(ctx.getString(R.string.json_key_bookId))
                    bookRelation.songId =
                        bookRelationsJSONObject.getInt(ctx.getString(R.string.json_key_songId))
                    bookRelation.page =
                        bookRelationsJSONObject.getString(ctx.getString(R.string.json_key_page))

                    bookRelationDao.insert(bookRelation)
                } catch (e: JSONException) {
                    DatabaseUpdateException(e.message).notifyUser(ctx)
                }
            }
            onCompleteCallback.invoke()
        }

    }
}