package de.thefeiter.liedgutverzeichnis.api.updater

import android.content.Context
import de.thefeiter.liedgutverzeichnis.R
import de.thefeiter.liedgutverzeichnis.api.loader.LoaderArray
import de.thefeiter.liedgutverzeichnis.exceptions.DatabaseUpdateException
import de.thefeiter.liedgutverzeichnis.objects.Category
import de.thefeiter.liedgutverzeichnis.roomdb.AppDb
import kotlinx.coroutines.DelicateCoroutinesApi
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.launch
import org.json.JSONArray
import org.json.JSONException

class UpdaterCategory(
    ctx: Context,
    db: AppDb,
    responseCallback: () -> Unit,
    completeCallback: () -> Unit
) : LoaderArray(
    ctx,
    db,
    responseCallback,
    completeCallback,
    ctx.getString(R.string.api_base_url) + ctx.getString(R.string.endpoint_categories)
) {
    @OptIn(DelicateCoroutinesApi::class)
    override suspend fun onResponse(response: JSONArray) {
        onResponseCallback.invoke()
        GlobalScope.launch {
            val categoryDao = db.categoryDao()
            categoryDao.deleteAll()
            for (i in 0 until response.length()) {
                try {
                    val category = response.getJSONObject(i)
                    val cat = Category()
                    cat.id = category.getInt(ctx.getString(R.string.json_key_categoryId))
                    cat.name = category.getString(ctx.getString(R.string.json_key_categoryName))
                    cat.color = category.getString(ctx.getString(R.string.json_key_categoryColor))
                    categoryDao.insert(cat)
                } catch (e: JSONException) {
                    DatabaseUpdateException(e.message).notifyUser(ctx)
                }
            }
            onCompleteCallback.invoke()
        }

    }
}