package de.thefeiter.liedgutverzeichnis.api.updater

import android.content.Context
import de.thefeiter.liedgutverzeichnis.R
import de.thefeiter.liedgutverzeichnis.api.loader.LoaderArray
import de.thefeiter.liedgutverzeichnis.exceptions.DatabaseUpdateException
import de.thefeiter.liedgutverzeichnis.objects.CategoryRelation
import de.thefeiter.liedgutverzeichnis.roomdb.AppDb
import kotlinx.coroutines.DelicateCoroutinesApi
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.launch
import org.json.JSONArray
import org.json.JSONException

class UpdaterCategoryRelations(
    ctx: Context,
    db: AppDb,
    responseCallback: () -> Unit,
    completeCallback: () -> Unit
) :
    LoaderArray(
        ctx,
        db,
        responseCallback,
        completeCallback,
        ctx.getString(R.string.api_base_url) + ctx.getString(R.string.endpoint_category_relation)
    ) {
    @OptIn(DelicateCoroutinesApi::class)
    override suspend fun onResponse(response: JSONArray) {
        onResponseCallback.invoke()
        GlobalScope.launch {
            val categoryRelationDao = db.categoryRelationDao()
            categoryRelationDao.deleteAll()
            for (i in 0 until response.length()) {
                try {
                    val category = response.getJSONObject(i)
                    val categoryRelation = CategoryRelation()
                    categoryRelation.categoryId =
                        category.getInt(ctx.getString(R.string.json_key_categoryId))
                    categoryRelation.songId =
                        category.getInt(ctx.getString(R.string.json_key_songId))
                    categoryRelationDao.insert(categoryRelation)
                } catch (e: JSONException) {
                    DatabaseUpdateException(e.message).notifyUser(ctx)
                }
            }
            onCompleteCallback.invoke()
        }

    }
}