package de.thefeiter.liedgutverzeichnis.api.updater

import android.content.Context
import de.thefeiter.liedgutverzeichnis.R
import de.thefeiter.liedgutverzeichnis.api.loader.LoaderArray
import de.thefeiter.liedgutverzeichnis.exceptions.DatabaseUpdateException
import de.thefeiter.liedgutverzeichnis.objects.Song
import de.thefeiter.liedgutverzeichnis.roomdb.AppDb
import kotlinx.coroutines.DelicateCoroutinesApi
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.launch
import org.json.JSONArray
import org.json.JSONException

class UpdaterSong(
    ctx: Context,
    db: AppDb,
    responseCallback: () -> Unit,
    completeCallback: () -> Unit
) : LoaderArray(
    ctx,
    db,
    responseCallback,
    completeCallback,
    ctx.getString(R.string.api_base_url) + ctx.getString(R.string.endpoint_song)
) {
    @OptIn(DelicateCoroutinesApi::class)
    override suspend fun onResponse(response: JSONArray) {
        onResponseCallback.invoke()
        GlobalScope.launch {
            val songDao = db.songDao()
            songDao.deleteAll()
            val song = Song()
            for (i in 0 until response.length()) {
                try {
                    val category = response.getJSONObject(i)
                    song.id = category.getInt(ctx.getString(R.string.json_key_songId))
                    song.title = category.getString(ctx.getString(R.string.json_key_title))
                    song.beginning = category.getString(ctx.getString(R.string.json_key_beginning))
                    song.keywords = category.getString(ctx.getString(R.string.json_key_keywords))
                    if (user.superuser) {
                        song.pdfId = category.getInt(ctx.getString(R.string.json_key_pdfId))
                    }
                    songDao.insert(song)
                } catch (e: JSONException) {
                    DatabaseUpdateException(e.message).notifyUser(ctx)
                }
            }
            onCompleteCallback.invoke()
        }

    }
}