package de.thefeiter.liedgutverzeichnis.api.updater

import android.content.Context
import de.thefeiter.liedgutverzeichnis.R
import de.thefeiter.liedgutverzeichnis.api.loader.LoaderArray
import de.thefeiter.liedgutverzeichnis.exceptions.DatabaseUpdateException
import de.thefeiter.liedgutverzeichnis.objects.SongSuggestion
import de.thefeiter.liedgutverzeichnis.roomdb.AppDb
import kotlinx.coroutines.DelicateCoroutinesApi
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.launch
import org.json.JSONArray
import org.json.JSONException

class UpdaterSongSuggestions(
    ctx: Context,
    db: AppDb,
    responseCallback: () -> Unit,
    completeCallback: () -> Unit
) :
    LoaderArray(
        ctx,
        db,
        responseCallback,
        completeCallback,
        ctx.getString(R.string.api_base_url) + ctx.getString(R.string.endpoint_song_suggestions)
    ) {
    @OptIn(DelicateCoroutinesApi::class)
    override suspend fun onResponse(response: JSONArray) {
        onResponseCallback.invoke()
        GlobalScope.launch {
            db.songSuggestionDao().deleteAll()
            for (i in 0 until response.length()) {
                try {
                    val songSuggestionJson = response.getJSONObject(i)
                    val songSuggestion = SongSuggestion()
                    songSuggestion.songId =
                        songSuggestionJson.getInt(ctx.getString(R.string.json_key_songId))
                    songSuggestion.suggestedSongId =
                        songSuggestionJson.getInt(ctx.getString(R.string.json_key_suggestedSongId))
                    db.songSuggestionDao().insert(songSuggestion)

                } catch (e: JSONException) {
                    DatabaseUpdateException(e.message).notifyUser(ctx)
                }
            }
            onCompleteCallback.invoke()
        }

    }
}