package de.thefeiter.liedgutverzeichnis.helpers;


import android.content.Context;
import android.view.View;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import de.thefeiter.liedgutverzeichnis.R;
import de.thefeiter.liedgutverzeichnis.objects.Song;
import de.thefeiter.liedgutverzeichnis.objects.User;
import de.thefeiter.liedgutverzeichnis.roomdb.AppDb;

public class SearchHelper {

    final User user;
    private final Context context;
    private final AppDb db;

    public SearchHelper(AppDb db, Context context) {
        this.context = context;
        this.db = db;

        user = new User();
    }

    public List<Song> search(String queryString, View view) {

        if (queryString.length() < 3) {
            new SnackbarHelper(view).makeSnackbar(context.getString(R.string.errors_search_query_short));
            return new ArrayList<>();
        }

        int activeBookCount = db.bookSettingDao().getActiveCount();

        if (activeBookCount == 0) {
            new SnackbarHelper(view).makeNoBookSnackbar();
            return new ArrayList<>();
        }

        List<Song> songs = db.songDao().getAll();

        List<Song> found = new ArrayList<>();


        for (int i = 0; i < songs.size(); i++) {
            String title = songs.get(i).title;
            String beginning = songs.get(i).beginning;
            String keyword = songs.get(i).keywords;


            float titleScore = triCompare(queryString, title);
            float beginningScore = triCompare(queryString, beginning);
            float keywordScore = triCompare(queryString, keyword);

            int score = (int) (Math.max(titleScore, Math.max(keywordScore, beginningScore)) * 100.0);


            if (score > 15) {
                de.thefeiter.liedgutverzeichnis.objects.Song foundSong = songs.get(i);

                foundSong.score = score;
                found.add(foundSong);
            }
        }

        // sort array
        found.sort(Comparator.comparingDouble(a -> a.score));

        Collections.reverse(found); // reverse order

        if (found.size() > 20) {
            found = found.subList(0, 20); //limit to 10 results
        }


        List<Song> foundSongs = new ConfigFilterHelper().filterSongsByConfig(db, found);


        if (foundSongs.size() == 0) {
            new SnackbarHelper(view).makeSnackbar(context.getString(R.string.search_no_song_found));
        }

        return foundSongs;
    }


    /**
     * Calculates how good the query string matches the data string. Return Value of 1.0 stands for query==data
     */
    private float triCompare(String query, String data) {

        if (data == null) return 0;

        if (data.length() < 3) return 0;


        query = query.toLowerCase();
        data = data.toLowerCase();

        if (data.equals(query)) return 1;

        String[] queryTrigrams = {};
        float score = 0;

        for (int i = 0; //split up query into trigrams
             i < (query.length() - 2);
             i++) {
            String trig = query.substring(i, i + 3);

            queryTrigrams = Arrays.copyOf(queryTrigrams, queryTrigrams.length + 1);

            queryTrigrams[queryTrigrams.length - 1] = trig;
        }


        for (String queryTrigram : queryTrigrams) {
            if (data.contains(queryTrigram)) {
                score++;
            }
        }

        score = score / queryTrigrams.length;

        return score;
    }

}
