package de.thefeiter.liedgutverzeichnis.helpers

import de.thefeiter.liedgutverzeichnis.objects.Song
import de.thefeiter.liedgutverzeichnis.roomdb.AppDb

class SongPathHelper(val db: AppDb) {

    fun findSongPath(targets: List<Song>, start: Song, max_depth: Int = 5): List<List<Song>> {

        val validPaths = mutableListOf<List<Song>>()

        fun findPaths(path: MutableList<Song>, depth: Int) {
            if (depth == max_depth) {
                return
            }

            val suggestions = db.songSuggestionDao().findAsSongBySongIdBidirectional(path.last().id)

            suggestions.forEach { suggestion ->
                if (targets.any { it.id == suggestion.id } and (suggestion.id != start.id)) { //found a target and suggestion is not the start itself
                    val validPath = path.toMutableList()
                    validPath.add(suggestion)
                    validPaths.add(validPath)
                    return@forEach
                }

                if (path.any { it.id == suggestion.id }) { // avoid duplicates and loops
                    return@forEach
                }

                path.add(suggestion)

                findPaths(path, depth + 1)
            }

        }

        findPaths(mutableListOf(start), 0)

        return validPaths.toList()
    }
}