package de.thefeiter.liedgutverzeichnis.helpers

import android.app.PendingIntent
import android.content.Context
import android.content.Intent
import android.content.SharedPreferences
import androidx.appcompat.app.AppCompatActivity
import androidx.core.app.NotificationCompat
import androidx.core.app.NotificationManagerCompat
import de.thefeiter.liedgutverzeichnis.MainActivity
import de.thefeiter.liedgutverzeichnis.R
import de.thefeiter.liedgutverzeichnis.api.ApiTest
import de.thefeiter.liedgutverzeichnis.api.loader.LoaderArray
import de.thefeiter.liedgutverzeichnis.objects.User
import de.thefeiter.liedgutverzeichnis.roomdb.AppDb
import de.thefeiter.liedgutverzeichnis.ui.settings.PREFERENCE_NAME
import de.thefeiter.liedgutverzeichnis.ui.settings.SHARED_PREFS_LAST_NOTIFICATION
import de.thefeiter.liedgutverzeichnis.ui.topflop.CatTopFlopActivity
import de.thefeiter.liedgutverzeichnis.ui.topflop.SongTopFlopActivity
import org.json.JSONArray
import kotlin.random.Random

const val tsDiff = 24 * 3600

class TopFlopNotificator(val context: Context, val db: AppDb) {

    private val sharedPreferences: SharedPreferences =
        context.getSharedPreferences(PREFERENCE_NAME, AppCompatActivity.MODE_PRIVATE)

    fun notifyUser() {
        val apiTest = ApiTest(context, true)
        apiTest.test(db) {

            object : LoaderArray(
                context, db, {}, {},
                context.getString(R.string.api_base_url) + context.getString(R.string.endpoint_topflop_category_get)
            ) {
                override suspend fun onResponse(response: JSONArray) {
                    if (response.length() > 0) makeCatNotification()
                }
            }.load()

            object : LoaderArray(
                context, db, {}, {},
                context.getString(R.string.api_base_url) + context.getString(R.string.endpoint_topflop_song_get)
            ) {
                override suspend fun onResponse(response: JSONArray) {
                    if (response.length() > 0) makeSongNotification()
                }
            }.load()

        }
    }

    fun checkAndNotify() {
        if (!User().allowTopflop) {
            return
        }
        val lastNotificationTs = sharedPreferences.getInt(SHARED_PREFS_LAST_NOTIFICATION, 0)
        val currentTs = MainActivity.timestamp.toInt()

        if (currentTs - lastNotificationTs > tsDiff) {
            notifyUser()
        }

        sharedPreferences.edit()
            .putInt(SHARED_PREFS_LAST_NOTIFICATION, currentTs)
            .apply()
    }

    fun makeCatNotification() {
        // Create an explicit intent for an Activity in your app
        val intent = Intent(context, CatTopFlopActivity::class.java).apply {
            flags = Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_CLEAR_TASK
        }
        val pendingIntent: PendingIntent = PendingIntent.getActivity(
            context, 0, intent,
            PendingIntent.FLAG_IMMUTABLE
        )

        val builder =
            NotificationCompat.Builder(context, MainActivity.CHANNEL_ID_TOPFLOP_NOTIFICATION)
                .setSmallIcon(R.drawable.ic_baseline_thumbs_up_down_black_24dp)
                .setContentTitle(context.getString(R.string.notification_new_categorySuggestions_title))
                .setContentText(context.getString(R.string.notification_new_categorySuggestions_text))
                .setPriority(NotificationCompat.PRIORITY_DEFAULT)
                .setContentIntent(pendingIntent)
                .setAutoCancel(true)


        with(NotificationManagerCompat.from(context)) {
            // notificationId is a unique int for each notification that you must define
            notify(Random.nextInt(), builder.build())
        }

    }

    fun makeSongNotification() {
        // Create an explicit intent for an Activity in your app
        val intent = Intent(context, SongTopFlopActivity::class.java).apply {
            flags = Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_CLEAR_TASK
        }
        val pendingIntent: PendingIntent = PendingIntent.getActivity(
            context, 0, intent,
            PendingIntent.FLAG_IMMUTABLE
        )


        val builder =
            NotificationCompat.Builder(context, MainActivity.CHANNEL_ID_TOPFLOP_NOTIFICATION)
                .setSmallIcon(R.drawable.ic_baseline_thumbs_up_down_black_24dp)
                .setContentTitle(context.getString(R.string.notification_new_songSuggestions_title))
                .setContentText(context.getString(R.string.notification_new_songSuggestions_text))
                .setPriority(NotificationCompat.PRIORITY_DEFAULT)
                .setContentIntent(pendingIntent)
                .setAutoCancel(true)

        with(NotificationManagerCompat.from(context)) {
            // notificationId is a unique int for each notification that you must define
            notify(Random.nextInt(), builder.build())
        }
    }
}