package de.thefeiter.liedgutverzeichnis.roomdb;

import android.database.Cursor;

import androidx.room.AutoMigration;
import androidx.room.Database;
import androidx.room.DeleteColumn;
import androidx.room.RoomDatabase;
import androidx.room.migration.AutoMigrationSpec;
import androidx.room.migration.Migration;
import androidx.sqlite.db.SupportSQLiteDatabase;

import de.thefeiter.liedgutverzeichnis.objects.Book;
import de.thefeiter.liedgutverzeichnis.objects.BookRelation;
import de.thefeiter.liedgutverzeichnis.objects.BookSetting;
import de.thefeiter.liedgutverzeichnis.objects.Category;
import de.thefeiter.liedgutverzeichnis.objects.CategoryRelation;
import de.thefeiter.liedgutverzeichnis.objects.HistoryElement;
import de.thefeiter.liedgutverzeichnis.objects.Song;
import de.thefeiter.liedgutverzeichnis.objects.SongPathSong;
import de.thefeiter.liedgutverzeichnis.objects.SongSuggestion;
import de.thefeiter.liedgutverzeichnis.objects.SuggestionCache;
import de.thefeiter.liedgutverzeichnis.objects.User;

@Database(
        entities = {
                Category.class,
                Song.class,
                CategoryRelation.class,
                Book.class,
                BookRelation.class,
                BookSetting.class,
                SongSuggestion.class,
                HistoryElement.class,
                SuggestionCache.class,
                SongPathSong.class},
        version = 38,
        autoMigrations = {
                @AutoMigration(from = 28, to = 29),
                @AutoMigration(from = 29, to = 30),
                @AutoMigration(from = 30, to = 31),
                @AutoMigration(from = 31, to = 32),
                @AutoMigration(from = 33, to = 34, spec = AppDb.AUTO_MIGRATION_SPEC_33_34.class),
                @AutoMigration(from = 35, to = 36),
                @AutoMigration(from = 36, to = 37),
                @AutoMigration(from = 37, to = 38),
        }
)
public abstract class AppDb extends RoomDatabase {

    public static final Migration MIGRATION_32_33 = new Migration(32, 33) {
        @Override
        public void migrate(SupportSQLiteDatabase database) {
            database.execSQL("UPDATE songs SET " +
                    "favorite = IFNULL((SELECT active from favorites WHERE favorites.songId = songs.songId LIMIT 1),0)," +
                    "favoriteUpdateTimestamp = IFNULL((SELECT updated from favorites WHERE favorites.songId = songs.songId   LIMIT 1),0)");
            database.execSQL("DROP TABLE favorites");
        }
    };

    @DeleteColumn(tableName = "bookSettings", columnName = "userId")
    static class AUTO_MIGRATION_SPEC_33_34 implements AutoMigrationSpec {
    }

    public static final Migration MIGRATION_34_35 = new Migration(34, 35) {
        @Override
        public void migrate(SupportSQLiteDatabase database) {
            Cursor c = database.query("SELECT api_token, superuser, allowTopFlop FROM users");
            c.moveToFirst();
            String token = c.getString(0);
            boolean superuser = c.getInt(1) == 1;
            boolean topflop = c.getInt(2) == 1;

            User user = new User();
            user.setApiToken(token);
            user.setSuperuser(superuser);
            user.setAllowTopflop(topflop);

            database.execSQL("DROP TABLE users");
        }
    };


    public abstract DAOCategory categoryDao();

    public abstract DAOSong songDao();

    public abstract DAOCategoryRelation categoryRelationDao();

    public abstract DAOBook bookDao();

    public abstract DAOBookRelation bookRelationDao();

    public abstract DAOBookSetting bookSettingDao();

    public abstract DAOSongSuggestion songSuggestionDao();

    public abstract DAOSongHistory historyDao();

    public abstract DAOSuggestionCache suggestionCacheDao();

    public abstract DAOSongPath songPathDao();

}
