package de.thefeiter.liedgutverzeichnis.roomdb

import androidx.room.Dao
import androidx.room.Insert
import androidx.room.Query
import de.thefeiter.liedgutverzeichnis.objects.Song

@Dao
interface DAOSong {
    @Query("SELECT * FROM songs")
    fun getAll(): List<Song>

    @Query("SELECT songs.* FROM songs WHERE songId = :songId LIMIT 1")
    fun findById(songId: Int): Song?

    @Query(
        "SELECT songs.* FROM songs " +
                "INNER JOIN bookRelations ON bookRelations.songId = songs.songId " +
                "INNER JOIN bookSettings on bookSettings.bookId = bookRelations.bookId " +
                "WHERE bookSettings.active ORDER BY RANDOM()"
    )
    fun loadRandomFilteredByUser(): Song?

    @Insert
    fun insert(song: Song)

    @Query("DELETE FROM songs")
    fun deleteAll()

    @Query(
        "SELECT DISTINCT songs.* FROM songs " +
                "INNER JOIN categoryRelations ON categoryRelations.songId = songs.songId " +
                "INNER JOIN bookRelations ON bookRelations.songId = songs.songId " +
                "INNER JOIN bookSettings on bookSettings.bookId = bookRelations.bookId " +
                "WHERE categoryRelations.categoryId = :categoryId AND bookSettings.active"
    )
    fun loadAllByCategoryFilteredByConfig(categoryId: Int): List<Song>

    @Query("SELECT songs.*, bookRelations.page FROM songs INNER JOIN bookRelations ON bookRelations.songId = songs.songId WHERE bookRelations.bookId = :bookId")
    fun loadAllByBookId(bookId: Int): List<Song>

    @Query(
        "SELECT DISTINCT " +
                "   songs.* " +
                "FROM songs " +
                "INNER JOIN bookRelations ON bookRelations.songId = songs.songId " +
                "INNER JOIN bookSettings on bookSettings.bookId = bookRelations.bookId " +
                "WHERE songs.songId IN (:songIds) AND bookSettings.active"
    )
    fun loadByIdsFilteredByConfig(songIds: IntArray?): List<Song>

    @Query("SELECT * FROM songs WHERE songId IN (:songIds)")
    fun loadAllByIds(songIds: IntArray?): List<Song>

    @Query(
        "SELECT DISTINCT " +
                "   songs.* " +
                "FROM songs " +
                "INNER JOIN bookRelations ON bookRelations.songId = songs.songId " +
                "INNER JOIN bookSettings on bookSettings.bookId = bookRelations.bookId " +
                "WHERE songs.songId NOT IN (SELECT DISTINCT categoryRelations.songId FROM categoryRelations) AND bookSettings.active"
    )
    fun loadUncategorized(): List<Song>

    @Query("UPDATE songs SET favorite = not favorite, favoriteUpdateTimestamp = :timestamp WHERE songId = :songId")
    fun toggleFavorite(songId: Int, timestamp: Double)

    @Query("SELECT favorite FROM songs WHERE songId = :songId LIMIT 1")
    fun isFavorite(songId: Int): Boolean

    @Query("SELECT * FROM songs WHERE songs.favorite")
    fun getAllFavorites(): List<Song>

    @Query("UPDATE songs SET favorite = :active, favoriteUpdateTimestamp = :timestamp WHERE songId = :songId")
    fun updateFavoriteState(songId: Int, active: Boolean, timestamp: Double)

    @Query("SELECT * FROM songs WHERE favoriteUpdateTimestamp > 0")
    fun getAllEverBeenFavorite(): List<Song>

    @Query("UPDATE songs SET bookmark = not bookmark, bookmarkUpdateTimestamp = :timestamp WHERE songId = :songId")
    fun toggleBookmark(songId: Int, timestamp: Double)

    @Query("SELECT bookmark FROM songs WHERE songId = :songId LIMIT 1")
    fun isBookmark(songId: Int): Boolean

    @Query("SELECT * FROM songs WHERE songs.bookmark")
    fun getAllBookmarks(): List<Song>

    @Query("SELECT * FROM songs WHERE bookmarkUpdateTimestamp > 0")
    fun getAllEverBeenBookmarked(): List<Song>

    @Query("UPDATE songs SET bookmark = :active, bookmarkUpdateTimestamp = :timestamp WHERE songId = :songId")
    fun updateBookmarkState(songId: Int, active: Boolean, timestamp: Double)
}