package de.thefeiter.liedgutverzeichnis.ui.dialogs

import android.app.Dialog
import android.content.Context
import android.content.DialogInterface
import android.os.Bundle
import android.widget.Toast
import androidx.appcompat.app.AlertDialog
import androidx.fragment.app.DialogFragment
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout
import de.thefeiter.liedgutverzeichnis.R
import de.thefeiter.liedgutverzeichnis.api.syncer.SyncerBookSettings
import de.thefeiter.liedgutverzeichnis.roomdb.AppDb
import de.thefeiter.liedgutverzeichnis.ui.settings.ActiveBooksAdapter
import de.thefeiter.liedgutverzeichnis.ui.views.Button

class BookSelectionDialog(val db: AppDb, val ctx: Context) : DialogFragment() {
    override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {

        // Use the Builder class for convenient dialog construction
        val builder = AlertDialog.Builder(ctx)

        // Get the layout inflater
        val inflater = this.layoutInflater
        val bookSelectDialog = inflater.inflate(R.layout.dialog_settings_bookselection, null)
        val recyclerView: RecyclerView = bookSelectDialog.findViewById(R.id.settingsRecyclerView)
        val invertButton: Button = bookSelectDialog.findViewById(R.id.bookSelection_btn_invert)
        val swipeRefreshLayout: SwipeRefreshLayout =
            bookSelectDialog.findViewById(R.id.settingsRefreshLayout)

        //set elements
        val books = db.bookDao().getAll()
        val adapter = ActiveBooksAdapter(books)

        recyclerView.layoutManager = LinearLayoutManager(ctx)
        recyclerView.adapter = adapter

        builder.setView(bookSelectDialog)
        builder.setPositiveButton(R.string.dialog_bookSelection_button_done) { dialog: DialogInterface?, id: Int -> }

        swipeRefreshLayout.setOnRefreshListener {
            SyncerBookSettings(ctx, db, {}) {
                Toast.makeText(ctx, R.string.sync_booksettings_complete, Toast.LENGTH_LONG).show()
                adapter.notifyDataSetChanged()
                swipeRefreshLayout.isRefreshing = false
            }.sync()
        }

        invertButton.setOnClickListener {
            adapter.invertSelection()
        }

        // Create the AlertDialog object and return it
        return builder.create()
    }
}