package de.thefeiter.liedgutverzeichnis.ui.dialogs;

import android.app.Activity;
import android.content.Context;
import android.graphics.Color;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;

import java.util.List;

import de.thefeiter.liedgutverzeichnis.R;
import de.thefeiter.liedgutverzeichnis.objects.Category;

public class CatSpinnerAdapter extends ArrayAdapter<Category> {

    private final List<Category> categories;
    private final LayoutInflater inflater;

    public CatSpinnerAdapter(Context context, int resource, List<Category> categories) {
        super(context, resource);
        this.categories = categories;
        this.inflater = (LayoutInflater) context.getSystemService(Activity.LAYOUT_INFLATER_SERVICE);

    }

    @Override
    public int getCount() {
        return categories.size();
    }

    @Override
    public Category getItem(int position) {
        return categories.get(position);
    }

    @Override
    public long getItemId(int position) {
        return getItem(position).id;
    }


    // And the "magic" goes here
    // This is for the "passive" state of the spinner
    @Override
    public View getView(int position, View convertView, ViewGroup parent) {

        if (convertView == null){
            convertView = inflater.inflate(R.layout.spinner_category_item_preview, parent, false);
        }


        TextView label = convertView.findViewById(R.id.spinner_category_item_preview_text);
        label.setText(getItem(position).name);

        ImageView icon = convertView.findViewById(R.id.spinner_category_item_preview_label);
        icon.setColorFilter(Color.parseColor(getItem(position).color));

        return convertView;
    }

    // And here is when the "chooser" is popped up
    // Normally is the same view, but you can customize it if you want
    @Override
    public View getDropDownView(int position, View convertView, @NonNull ViewGroup parent) {

        View row = inflater.inflate(R.layout.spinner_category_item, parent, false);
        TextView label = row.findViewById(R.id.spinner_category_item_text);
        label.setText(getItem(position).name);

        ImageView icon = row.findViewById(R.id.spinner_category_item_label);
        icon.setColorFilter(Color.parseColor(getItem(position).color));

        return row;
    }
}
