package de.thefeiter.liedgutverzeichnis.ui.dialogs

import android.app.Dialog
import android.content.Context
import android.os.Bundle
import android.widget.Spinner
import android.widget.Toast
import androidx.appcompat.app.AlertDialog
import androidx.core.content.ContextCompat
import androidx.fragment.app.DialogFragment
import de.thefeiter.liedgutverzeichnis.R
import de.thefeiter.liedgutverzeichnis.objects.Category
import de.thefeiter.liedgutverzeichnis.objects.Song
import de.thefeiter.liedgutverzeichnis.objects.SuggestionCache
import de.thefeiter.liedgutverzeichnis.roomdb.AppDb
import java.util.*

class CategorySuggestionDialog(
    private val db: AppDb,
    private val song: Song,
    private val ctx: Context,
    private val catNameComparator: Comparator<Category> = Comparator { c1: Category, c2: Category ->
        c1.name.compareTo(
            c2.name,
            ignoreCase = true
        )
    }
) : DialogFragment() {
    override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {


        // Use the Builder class for convenient dialog construction
        val builder = AlertDialog.Builder(ctx)

        // Get the layout inflater
        val inflater = requireActivity().layoutInflater
        val setupDialog = inflater.inflate(R.layout.dialog_category_suggestion, null)
        val spin = setupDialog.findViewById<Spinner>(R.id.spinner2)

        val existingCategories = db.categoryDao().findBySongId(song.id)
        val suggestableCategories: MutableList<Category> = ArrayList()
        for (category in db.categoryDao().getAll()) {
            var alreadyExists = false
            for (existingCategory in existingCategories) {
                alreadyExists = category.id == existingCategory.id || alreadyExists
            }
            if (!alreadyExists) suggestableCategories.add(category)
        }
        suggestableCategories.sortWith(catNameComparator)

        val defaultSelection = Category()
        defaultSelection.id = Category.DEFAULT_SELECTION
        defaultSelection.name = getString(R.string.dialog_category_suggestion_default_selection)
        defaultSelection.color =
            "#" + Integer.toHexString(ContextCompat.getColor(ctx, R.color.grey))
        suggestableCategories.add(0, defaultSelection)

        val adapter =
            CatSpinnerAdapter(ctx, R.layout.spinner_category_item_preview, suggestableCategories)
        adapter.setDropDownViewResource(R.layout.spinner_category_item)

        spin.adapter = adapter

        builder.setView(setupDialog)
        builder.setPositiveButton(R.string.dialog_suggestion_save) { _, _ ->
            // Test API token and save
            val selectedCat = spin.selectedItem as Category
            if (selectedCat.id == Category.DEFAULT_SELECTION) {
                Toast.makeText(
                    ctx,
                    R.string.dialog_category_suggestion_no_category,
                    Toast.LENGTH_SHORT
                ).show()
                return@setPositiveButton
            }

            val suggestionCache = SuggestionCache()

            suggestionCache.songId = song.id
            suggestionCache.categoryId = selectedCat.id


            db.suggestionCacheDao().insert(suggestionCache)
            Toast.makeText(ctx, R.string.suggestion_saved, Toast.LENGTH_SHORT).show()
        }
        builder.setNegativeButton(R.string.cancel) { _, _ -> dismiss() }

        // Create the AlertDialog object and return it
        return builder.create()
    }
}