package de.thefeiter.liedgutverzeichnis.ui.dialogs

import android.annotation.SuppressLint
import android.app.Dialog
import android.content.Context
import android.content.DialogInterface
import android.os.Bundle
import android.view.KeyEvent
import android.view.View
import android.view.inputmethod.EditorInfo
import android.view.inputmethod.InputMethodManager
import android.widget.Button
import android.widget.EditText
import android.widget.TextView
import android.widget.Toast
import androidx.appcompat.app.AlertDialog
import androidx.fragment.app.DialogFragment
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import de.thefeiter.liedgutverzeichnis.MainActivity
import de.thefeiter.liedgutverzeichnis.R
import de.thefeiter.liedgutverzeichnis.helpers.SearchHelper
import de.thefeiter.liedgutverzeichnis.objects.Song
import de.thefeiter.liedgutverzeichnis.objects.SuggestionCache
import de.thefeiter.liedgutverzeichnis.roomdb.AppDb
import de.thefeiter.liedgutverzeichnis.ui.main.AdapterSongList

class SongSuggestionDialog(
    private val db: AppDb,
    private val song: Song,
    private val ctx: Context
) : DialogFragment() {
    @SuppressLint("NotifyDataSetChanged")
    override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {


        // Use the Builder class for convenient dialog construction
        val builder = AlertDialog.Builder(
            ctx
        )

        // Get the layout inflater
        val inflater = requireActivity().layoutInflater
        val suggestionDialog = inflater.inflate(R.layout.fragment_search, null)
        val queryBox = suggestionDialog.findViewById<EditText>(R.id.searchQuery)
        val searchButton = suggestionDialog.findViewById<Button>(R.id.querySubmit)
        val resultRecyclerView: RecyclerView =
            suggestionDialog.findViewById(R.id.searchRecyclerView)

        // show search results in recycler view
        resultRecyclerView.layoutManager = LinearLayoutManager(ctx)
        val adapter = AdapterSongList(ArrayList()) { selectedSong: Song?, _: View? ->

            val suggestionCache = SuggestionCache()

            suggestionCache.songId = song.id
            suggestionCache.suggestedSongId = selectedSong!!.id


            db.suggestionCacheDao().insert(suggestionCache)
            Toast.makeText(ctx, R.string.suggestion_saved, Toast.LENGTH_SHORT).show()
            dismiss()
        }
        resultRecyclerView.adapter = adapter


        // if search button clicked, start search
        searchButton.setOnClickListener {
            val sp = SearchHelper(MainActivity.getAppDb(), ctx)
            adapter.songs = sp.search(queryBox.text.toString(), suggestionDialog)
            adapter.notifyDataSetChanged()

            //hide keyboard
            val manager =
                ctx.getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
            manager.hideSoftInputFromWindow(suggestionDialog.windowToken, 0)
        }


        //If submit via keyboard, click search button
        queryBox.setOnEditorActionListener { _: TextView?, actionId: Int, _: KeyEvent? ->
            if (actionId == EditorInfo.IME_ACTION_NEXT || actionId == EditorInfo.IME_ACTION_DONE) {
                searchButton.performClick()
                return@setOnEditorActionListener true
            }
            false
        }
        builder.setView(suggestionDialog)
        builder.setTitle(R.string.dialog_song_suggestion_title)
        builder.setNegativeButton(R.string.cancel) { _: DialogInterface?, _: Int -> dismiss() }

        // Create the AlertDialog object and return it
        return builder.create()
    }
}