package de.thefeiter.liedgutverzeichnis.ui.dialogs

import android.app.Dialog
import android.content.Context
import android.content.DialogInterface
import android.os.Bundle
import android.view.View
import android.widget.EditText
import android.widget.Toast
import androidx.appcompat.app.AlertDialog
import androidx.fragment.app.DialogFragment
import androidx.fragment.app.FragmentManager
import de.thefeiter.liedgutverzeichnis.R
import de.thefeiter.liedgutverzeichnis.api.UserManager
import de.thefeiter.liedgutverzeichnis.helpers.SnackbarHelper
import de.thefeiter.liedgutverzeichnis.helpers.SyncHelper
import de.thefeiter.liedgutverzeichnis.roomdb.AppDb

class UserRegisterDialog(
    val db: AppDb,
    val ctx: Context,
    private val supportFragmentManager: FragmentManager,
    private val syncHelper: SyncHelper,
    private val snackbarView: View? = null,
    private val showHelpSnackbar: Boolean = false
) : DialogFragment() {
    override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {

        // Use the Builder class for convenient dialog construction
        val builder = AlertDialog.Builder(ctx)

        // Get the layout inflater
        val inflater = requireActivity().layoutInflater
        val setupDialog = inflater.inflate(R.layout.dialog_register, null)
        val usernameInput = setupDialog.findViewById<EditText>(R.id.user_name)
        builder.setView(setupDialog)
        builder.setPositiveButton(R.string.register) { _: DialogInterface?, _: Int ->
            // Test API token and save
            val username = usernameInput.text.toString()

            if (username.length < 5) {
                Toast.makeText(ctx, R.string.username_too_short, Toast.LENGTH_LONG).show()
            } else {
                //test api token
                val userManager = UserManager(
                    ctx, db, supportFragmentManager, syncHelper
                )
                userManager.registerUser(username)
            }


        }

        builder.setNegativeButton(R.string.cancel) { _: DialogInterface?, _: Int ->
            // Notify and close the App
            Toast.makeText(ctx, R.string.no_update_without_token, Toast.LENGTH_LONG).show()
        }

        // Create the AlertDialog object and return it
        return builder.create()
    }

    fun show() {
        super.show(supportFragmentManager, null)
    }

    override fun onDestroy() {
        super.onDestroy()

        snackbarView?.let {
            if (showHelpSnackbar) {
                SnackbarHelper(it).makeHelpSnackbar()
            }
        }

    }
}