package de.thefeiter.liedgutverzeichnis.ui.dialogs

import android.app.Dialog
import android.content.Context
import android.content.DialogInterface
import android.os.Bundle
import android.view.View
import android.widget.EditText
import android.widget.Toast
import androidx.appcompat.app.AlertDialog
import androidx.fragment.app.DialogFragment
import androidx.fragment.app.FragmentManager
import de.thefeiter.liedgutverzeichnis.R
import de.thefeiter.liedgutverzeichnis.api.UserManager
import de.thefeiter.liedgutverzeichnis.helpers.SnackbarHelper
import de.thefeiter.liedgutverzeichnis.helpers.SyncHelper
import de.thefeiter.liedgutverzeichnis.roomdb.AppDb

class UserSetupDialog(
    val db: AppDb,
    val ctx: Context,
    private val supportFragmentManager: FragmentManager,
    private val syncHelper: SyncHelper,
    private val snackbarView: View? = null,
    private var showHelpSnackbar: Boolean = false
) : DialogFragment() {
    override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {

        // Use the Builder class for convenient dialog construction
        val builder = AlertDialog.Builder(
            ctx
        )

        // Get the layout inflater
        val inflater = requireActivity().layoutInflater
        val setupDialog = inflater.inflate(R.layout.dialog_setup, null)
        val apiTokenInput = setupDialog.findViewById<EditText>(R.id.setup_token)
        builder.setView(setupDialog)
        builder.setPositiveButton(R.string.save_token) { dialog: DialogInterface?, id: Int ->
            // Test API token and save
            val token = apiTokenInput.text.toString()

            //test api token
            val userManager = UserManager(
                ctx, db, supportFragmentManager, syncHelper
            )
            userManager.testToken(token)
        }
        builder.setNegativeButton(R.string.cancel) { _: DialogInterface?, _: Int ->
            // Notify and close the App
            Toast.makeText(ctx, R.string.no_update_without_token, Toast.LENGTH_LONG).show()
        }
        builder.setNeutralButton(R.string.register_new_token) { _: DialogInterface?, _: Int ->
            showHelpSnackbar = false
            val setupDialog = UserRegisterDialog(
                db,
                ctx,
                supportFragmentManager,
                syncHelper,
                snackbarView,
                true
            )
            setupDialog.show()

        }

        // Create the AlertDialog object and return it
        return builder.create()
    }

    fun show() {
        super.show(supportFragmentManager, null)
    }

    override fun onDestroy() {
        super.onDestroy()

        snackbarView?.let {
            if (showHelpSnackbar) {
                SnackbarHelper(it).makeHelpSnackbar()
            }
        }

    }
}