package de.thefeiter.liedgutverzeichnis.ui.dialogs.songpathselection

import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.FrameLayout
import android.widget.TextView
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import de.thefeiter.liedgutverzeichnis.MainActivity.Companion.getAppDb
import de.thefeiter.liedgutverzeichnis.R
import de.thefeiter.liedgutverzeichnis.objects.Song
import de.thefeiter.liedgutverzeichnis.roomdb.AppDb
import de.thefeiter.liedgutverzeichnis.ui.dialogs.songsequencesuggestion.AdapterSongPill
import de.thefeiter.liedgutverzeichnis.ui.dialogs.songsequencesuggestion.SongWithArrow

class AdapterSongPath(
    private val songPaths: List<List<Song>>,
    private val context: Context,
    private val onPathChoose: (List<Song>) -> Unit
) :
    RecyclerView.Adapter<AdapterSongPath.ViewHolder>() {
    private val db: AppDb = getAppDb()
    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
        val layoutInflater = LayoutInflater.from(parent.context)
        val listItem = layoutInflater.inflate(R.layout.item_song_path, parent, false)
        return ViewHolder(listItem)
    }

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {


        val songsWithArrow = ArrayList<SongWithArrow>()

        for (song in songPaths[position]) {
            songsWithArrow.add(SongWithArrow(song, false))
            if (songPaths[position].indexOf(song) != songPaths[position].size - 1) {
                songsWithArrow.add(SongWithArrow(Song(), true))
            }
        }

        holder.recycler.adapter = AdapterSongPill(songsWithArrow, db, true)

        holder.recycler.setHasFixedSize(true)
        holder.recycler.layoutManager =
            LinearLayoutManager(context, LinearLayoutManager.HORIZONTAL, false)


        holder.title.text =
            context.getString(R.string.song_path_target_title, songPaths[position].last().title)

        holder.button.setOnClickListener {

            onPathChoose.invoke(songPaths[position])


        }


    }


    override fun getItemCount(): Int {
        return songPaths.size
    }

    class ViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        val recycler: RecyclerView = itemView.findViewById(R.id.pathSongRecycler)
        val layout: View = itemView.findViewById(R.id.songPathItemLayout)
        val title: TextView = itemView.findViewById(R.id.songPathTitle)
        val button: FrameLayout = itemView.findViewById(R.id.songPathChoose)

    }

}