package de.thefeiter.liedgutverzeichnis.ui.dialogs.songpathselection

import android.app.Dialog
import android.content.Context
import android.content.DialogInterface
import android.os.Bundle
import androidx.appcompat.app.AlertDialog
import androidx.fragment.app.DialogFragment
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import de.thefeiter.liedgutverzeichnis.R
import de.thefeiter.liedgutverzeichnis.objects.Song
import de.thefeiter.liedgutverzeichnis.objects.SongPathSong
import de.thefeiter.liedgutverzeichnis.roomdb.AppDb

class SongPathSelectionDialog(
    val db: AppDb,
    val ctx: Context,
    private val songPaths: List<List<Song>>,
    private val onNewPathChosen: () -> Unit
) : DialogFragment() {
    override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {

        // Use the Builder class for convenient dialog construction
        val builder = AlertDialog.Builder(ctx)

        // Get the layout inflater
        val inflater = this.layoutInflater
        val bookSelectDialog = inflater.inflate(R.layout.dialog_song_path_selection, null)
        val recyclerView: RecyclerView = bookSelectDialog.findViewById(R.id.songPathRecycler)


        val adapter = AdapterSongPath(songPaths, ctx) { chosenSongPath ->

            //when path chosen
            db.songPathDao().deleteAll()

            //remove start song
            val path = chosenSongPath.toMutableList()
            path.removeAt(0)

            var index = 0
            path.forEach {
                val sps = SongPathSong()
                sps.index = index
                sps.songId = it.id
                db.songPathDao().insert(sps)
                index++
            }

            onNewPathChosen.invoke()
            dismiss()

        }

        recyclerView.layoutManager = LinearLayoutManager(ctx)
        recyclerView.adapter = adapter

        builder.setView(bookSelectDialog)
        builder.setNegativeButton(R.string.cancel) { _: DialogInterface?, _: Int -> }
        builder.setTitle(getString(R.string.dialog_song_path_title))


        // Create the AlertDialog object and return it
        return builder.create()
    }
}