package de.thefeiter.liedgutverzeichnis.ui.dialogs.songsequencesuggestion

import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.RecyclerView
import de.thefeiter.liedgutverzeichnis.R
import de.thefeiter.liedgutverzeichnis.objects.Song
import de.thefeiter.liedgutverzeichnis.roomdb.AppDb
import de.thefeiter.liedgutverzeichnis.ui.views.SongPill

class SongWithArrow(
    val song: Song,
    val isArrow: Boolean
)

class AdapterSongPill(
    val listElements: List<SongWithArrow>,
    private val db: AppDb,
    private val isHorizontal: Boolean = false
) :
    RecyclerView.Adapter<RecyclerView.ViewHolder>() {

    private var arrowResource = R.layout.item_arrow_down
    private var pillResource = R.layout.item_song_pill_vertical

    companion object {
        const val SONG_PILL = 0
        const val ARROW = 1
    }

    init {
        if (isHorizontal) {
            arrowResource = R.layout.item_arrow_right
            pillResource = R.layout.item_song_pill_horizontal
        }
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
        val layoutInflater = LayoutInflater.from(parent.context)

        if (viewType == ARROW) {
            return ArrowViewHolder(
                layoutInflater.inflate(
                    arrowResource,
                    parent,
                    false
                )
            )
        }

        return SongViewHolder(
            layoutInflater.inflate(
                pillResource,
                parent,
                false
            )
        )
    }

    override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {

        if (getItemViewType(position) == ARROW) {
            (holder as ArrowViewHolder).bind(position)
        } else {
            (holder as SongViewHolder).bind(position)
        }

    }

    override fun getItemCount(): Int {
        return listElements.size
    }

    override fun getItemViewType(position: Int): Int {
        return if (listElements[position].isArrow) ARROW else SONG_PILL
    }


    private inner class SongViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        val songPill: SongPill = itemView.findViewById(R.id.song_pill)
        fun bind(position: Int) {
            val listElement = listElements[position]

            songPill.label.text = listElement.song.title

            songPill.setNoRipple()
        }
    }


    private inner class ArrowViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        fun bind(position: Int) {


        }
    }


}

