package de.thefeiter.liedgutverzeichnis.ui.dialogs.songsequencesuggestion

import android.app.Dialog
import android.content.Context
import android.os.Bundle
import android.widget.Toast
import androidx.appcompat.app.AlertDialog
import androidx.fragment.app.DialogFragment
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import de.thefeiter.liedgutverzeichnis.R
import de.thefeiter.liedgutverzeichnis.objects.Song
import de.thefeiter.liedgutverzeichnis.objects.SuggestionCache
import de.thefeiter.liedgutverzeichnis.roomdb.AppDb

class SongSequenceSuggestionDialog(
    private val db: AppDb,
    private val songs: List<Song>,
    private val ctx: Context
) : DialogFragment() {
    override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {


        // Use the Builder class for convenient dialog construction
        val builder = AlertDialog.Builder(ctx)

        // Get the layout inflater
        val inflater = requireActivity().layoutInflater
        val setupDialog = inflater.inflate(R.layout.dialog_song_sequence_suggestion, null)
        val recyclerView: RecyclerView =
            setupDialog.findViewById(R.id.song_suggestion_sequence_recycler)

        val songsWithArrow = ArrayList<SongWithArrow>()

        for (song in songs) {
            songsWithArrow.add(SongWithArrow(song, false))
            if (songs.indexOf(song) != songs.size - 1) {
                songsWithArrow.add(SongWithArrow(Song(), true))
            }
        }

        val adapter = AdapterSongPill(songsWithArrow, db)
        recyclerView.setHasFixedSize(true)
        recyclerView.layoutManager = LinearLayoutManager(ctx)
        recyclerView.adapter = adapter

        builder.setView(setupDialog)
        builder.setPositiveButton(R.string.dialog_suggestion_save) { _, _ ->

            for (song in songs) {
                if (songs.indexOf(song) != songs.size - 1) {

                    val suggestionCache = SuggestionCache()

                    suggestionCache.songId = song.id
                    suggestionCache.suggestedSongId = songs[songs.indexOf(song) + 1].id


                    db.suggestionCacheDao().insert(suggestionCache)
                }
            }
            Toast.makeText(ctx, R.string.suggestion_saved, Toast.LENGTH_SHORT).show()

            dismiss()
        }
        builder.setNegativeButton(R.string.cancel) { _, _ -> dismiss() }

        // Create the AlertDialog object and return it
        return builder.create()
    }
}