package de.thefeiter.liedgutverzeichnis.ui.main

import android.content.Intent
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.RelativeLayout
import android.widget.TextView
import androidx.recyclerview.widget.RecyclerView
import de.thefeiter.liedgutverzeichnis.BOOK_ID
import de.thefeiter.liedgutverzeichnis.MainActivity
import de.thefeiter.liedgutverzeichnis.MainActivity.Companion.getAppDb
import de.thefeiter.liedgutverzeichnis.MainActivity.Companion.timestamp
import de.thefeiter.liedgutverzeichnis.R
import de.thefeiter.liedgutverzeichnis.objects.HistorySong
import de.thefeiter.liedgutverzeichnis.roomdb.AppDb
import de.thefeiter.liedgutverzeichnis.ui.songdetails.SongDetails
import java.text.DateFormat.getDateInstance
import java.util.*
import kotlin.collections.ArrayList
import kotlin.math.roundToInt

const val THRESHOLD_DISPLAY_HOURS = 1
const val THRESHOLD_DISPLAY_DAYS = 24
const val THRESHOLD_DISPLAY_DATE = 24 * 7

class HistorySongWithDelta(song: HistorySong) : HistorySong() {

    init {
        super.song = song.song
    }

    var timeDelta: Double = 0.0
}

class AdapterHistoryList(listElements: MutableList<HistorySong>) :
    RecyclerView.Adapter<RecyclerView.ViewHolder>() {

    var listElements: MutableList<HistorySongWithDelta> = ArrayList()
    private val db: AppDb = getAppDb()

    init {
        updateData(listElements)
    }

    fun updateData(listElements: MutableList<HistorySong>) {
        this.listElements = ArrayList()
        var nextHourDelta = 0
        listElements.forEach {
            val tsDelta = timestamp - it.timestamp

            val hourDelta = (tsDelta / 3600).roundToInt()
            val dayDelta: Int = (hourDelta / 24)

            if (hourDelta > nextHourDelta) {
                if (dayDelta >= 1) {
                    nextHourDelta = 24 * (dayDelta + 1)
                    val histSong = HistorySongWithDelta(HistorySong())
                    histSong.timeDelta = dayDelta * 3600.0 * 24.0
                    this.listElements.add(histSong)
                } else {
                    nextHourDelta = hourDelta + 1
                    val histSong = HistorySongWithDelta(HistorySong())
                    histSong.timeDelta = hourDelta * 3600.0
                    this.listElements.add(histSong)
                }
            }
            this.listElements.add(HistorySongWithDelta(it))
        }
    }

    companion object {
        const val SONG_ELEMENT = 0
        const val TIMESTAMP_ELEMENT = 1
    }


    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
        val layoutInflater = LayoutInflater.from(parent.context)

        if (viewType == TIMESTAMP_ELEMENT) {
            return TimestampViewHolder(
                layoutInflater.inflate(
                    R.layout.item_history_timestamp,
                    parent,
                    false
                )
            )
        }

        return SongViewHolder(layoutInflater.inflate(R.layout.item_songlist, parent, false))
    }

    override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {

        if (getItemViewType(position) == TIMESTAMP_ELEMENT) {
            (holder as TimestampViewHolder).bind(position)
        } else {
            (holder as SongViewHolder).bind(position)
        }

    }

    override fun getItemCount(): Int {
        return listElements.size
    }

    override fun getItemViewType(position: Int): Int {
        return if (listElements[position].timeDelta > 0) TIMESTAMP_ELEMENT else SONG_ELEMENT
    }


    private inner class SongViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        val songTitle: TextView = itemView.findViewById(R.id.songTitle)
        val songBeginning: TextView = itemView.findViewById(R.id.songBeginning)
        val bookPage: TextView = itemView.findViewById(R.id.book_page)
        val icon: ImageView = itemView.findViewById(R.id.songIcon)
        val favStar: ImageView = itemView.findViewById(R.id.fav_star)
        val relativeLayout: RelativeLayout = itemView.findViewById(R.id.songItemRelativeLayout)
        fun bind(position: Int) {
            val context = itemView.context

            val listElement = listElements[position]

            songTitle.text = listElement.song.title
            songBeginning.text = listElement.song.beginning

            //according to favorite state set visibility of star icon
            if (db.songDao().isFavorite(listElement.song.id)) {
                favStar.visibility = View.VISIBLE
            } else {
                favStar.visibility = View.INVISIBLE
            }
            if (listElement.song.page != null) {
                bookPage.text = listElement.song.page
            }

            //bei click on song
            relativeLayout.setOnClickListener {
                //start new activity with song object by intent
                val intent = Intent(context, SongDetails::class.java)
                intent.putExtra(MainActivity.SONG_DETAILS, listElement.song.id)
                if (listElement.song.bookId != 0) {
                    intent.putExtra(BOOK_ID, listElement.song.bookId)
                }
                context.startActivity(intent)
            }

            icon.setOnClickListener {
                val timestamp = timestamp
                //according to favorite state set visibility of star icon and toggle status
                db.songDao().toggleFavorite(listElement.song.id, timestamp)
                if (db.songDao().isFavorite(listElement.song.id)) {
                    favStar.visibility = View.VISIBLE
                } else {
                    favStar.visibility = View.INVISIBLE
                }
            }
        }
    }


    private inner class TimestampViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        val text: TextView = itemView.findViewById(R.id.historyTimestampText)

        fun bind(position: Int) {

            val hours = (listElements[position].timeDelta / 3600).toInt()

            text.text = getTimeString(hours)
        }

        fun getTimeString(delta_hours: Int): String {


            when {
                delta_hours < THRESHOLD_DISPLAY_HOURS -> {
                    return itemView.context.getString(R.string.history_time_just_now)
                }
                delta_hours < THRESHOLD_DISPLAY_DAYS -> {

                    val hours: Int = delta_hours / THRESHOLD_DISPLAY_HOURS

                    if (hours == 1) {
                        return itemView.context.getString(R.string.history_time_hours_single)
                    }
                    return itemView.context.getString(R.string.history_time_hours).format(hours)
                }
                delta_hours < THRESHOLD_DISPLAY_DATE -> {
                    val days: Int = delta_hours / THRESHOLD_DISPLAY_DAYS

                    if (days == 1) {
                        return itemView.context.getString(R.string.history_time_days_single)
                    }
                    return itemView.context.getString(R.string.history_time_days).format(days)
                }
                else -> {

                    val date =
                        Date((timestamp * 1000).toLong() - (delta_hours * 3600 * 1000).toLong()) // *1000 is to convert seconds to milliseconds
                    //val sdf  = //SimpleDateFormat("EEE, dd MMM yyyy ") // the format of your date
                    //sdf.setTimeZone(TimeZone.getTimeZone("GMT-4"))

                    return getDateInstance().format(date)
                }
            }
        }
    }


}

