package de.thefeiter.liedgutverzeichnis.ui.main;

import static de.thefeiter.liedgutverzeichnis.SongChooserKt.BOOK_ID;
import static de.thefeiter.liedgutverzeichnis.SongChooserKt.SORT_BY_BEGINNING;
import static de.thefeiter.liedgutverzeichnis.SongChooserKt.SORT_BY_PAGE;
import static de.thefeiter.liedgutverzeichnis.SongChooserKt.SORT_BY_TITLE;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.Intent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.qtalk.recyclerviewfastscroller.RecyclerViewFastScroller;

import org.jetbrains.annotations.NotNull;

import java.util.Comparator;
import java.util.List;
import java.util.Locale;

import de.thefeiter.liedgutverzeichnis.MainActivity;
import de.thefeiter.liedgutverzeichnis.R;
import de.thefeiter.liedgutverzeichnis.objects.Song;
import de.thefeiter.liedgutverzeichnis.roomdb.AppDb;
import de.thefeiter.liedgutverzeichnis.ui.songdetails.SongDetails;

public class AdapterSongList extends RecyclerView.Adapter<AdapterSongList.ViewHolder> implements RecyclerViewFastScroller.OnPopupTextUpdate {
    final Comparator<Song> titleComparator = (s1, s2) -> s1.title.compareToIgnoreCase(s2.title);
    final Comparator<Song> beginningComparator = (s1, s2) -> s1.beginning.compareToIgnoreCase(s2.beginning);
    final Comparator<Song> pageComparator = (s1, s2) -> (int) (stringValue(s1.page) - stringValue(s2.page));
    public List<Song> songs;
    private final AppDb db;
    public String sortBy;
    private OnClickListener onClickListener;


    // RecyclerView recyclerView;
    public AdapterSongList(List<Song> listData, @NonNull String sortBy) {
        this.songs = listData;
        this.db = MainActivity.getAppDb();
        this.sortBy = sortBy;
        //default OnClick
        this.onClickListener = (selectedSong, view) -> {
            Context context = view.getContext();

            //start new activity with song object by intent
            Intent intent = new Intent(context, SongDetails.class);
            intent.putExtra(MainActivity.SONG_DETAILS, selectedSong.id);


            if (selectedSong.bookId != 0) {
                intent.putExtra(BOOK_ID, selectedSong.bookId);
            }

            context.startActivity(intent);
        };

        //sort according to sortBy identifier
        sort(sortBy);
    }


    public AdapterSongList(List<Song> listData) {
        this(listData, SORT_BY_TITLE);
    }

    public AdapterSongList(List<Song> listData, OnClickListener onClickListener) {
        this(listData, SORT_BY_TITLE);
        this.onClickListener = onClickListener;
    }

    @NotNull
    @Override
    public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        LayoutInflater layoutInflater = LayoutInflater.from(parent.getContext());
        View listItem = layoutInflater.inflate(R.layout.item_songlist, parent, false);
        return new ViewHolder(listItem);
    }

    @Override
    public void onBindViewHolder(ViewHolder holder, int position) {
        final Song song = songs.get(position);
        holder.songTitle.setText(song.title);
        holder.songBeginning.setText(song.beginning);


        //according to favorite state set visibility of star icon
        if (db.songDao().isFavorite(song.id)) {
            holder.favStar.setVisibility(View.VISIBLE);
        } else {
            holder.favStar.setVisibility(View.INVISIBLE);
        }

        if (song.page != null) {
            holder.bookPage.setText(song.page);
        }


        //bei click on song
        holder.relativeLayout.setOnClickListener(view -> this.onClickListener.onClick(song, view));


        holder.icon.setOnClickListener(view -> {
            double timestamp = MainActivity.getTimestamp();
            //according to favorite state set visibility of star icon and toggle status
            db.songDao().toggleFavorite(song.id, timestamp);
            if (db.songDao().isFavorite(song.id)) {
                holder.favStar.setVisibility(View.VISIBLE);
            } else {
                holder.favStar.setVisibility(View.INVISIBLE);
            }
        });
    }


    @Override
    public int getItemCount() {
        return songs.size();
    }

    private long stringValue(String inString) {
        if (inString == null) return 0;
        long value = 0;
        for (int i = 0; i < inString.length(); i++) {
            char c = inString.charAt(i);
            long multiplicator = (long) Math.pow(10, inString.length() - i - 1);
            value = value + ((int) c * multiplicator);
        }
        return value;
    }

    @NonNull
    @Override
    public CharSequence onChange(int i) {
        // if stream library available sort according to sortBy identifier
        switch (sortBy) {
            case SORT_BY_TITLE:
                return songs.get(i).title.toUpperCase(Locale.ROOT).substring(0, 1);
            case SORT_BY_BEGINNING:
                return songs.get(i).beginning.toUpperCase(Locale.ROOT).substring(0, 1);
            case SORT_BY_PAGE:
                return songs.get(i).page;
            default:
                return "";
        }

    }

    @SuppressLint("NotifyDataSetChanged")
    public void sort(String sortBy) {
        this.sortBy = sortBy;
        // if stream library available sort according to sortBy identifier
        switch (sortBy) {
            case SORT_BY_TITLE:
                songs.sort(titleComparator);
                break;
            case SORT_BY_BEGINNING:
                songs.sort(beginningComparator);
                break;
            case SORT_BY_PAGE:
                songs.sort(pageComparator);
                break;
        }
        this.notifyDataSetChanged();

    }

    public static class ViewHolder extends RecyclerView.ViewHolder {
        public final TextView songTitle;
        public final TextView songBeginning;
        public final TextView bookPage;
        public final ImageView icon;
        public final ImageView favStar;
        public final RelativeLayout relativeLayout;

        public ViewHolder(View itemView) {
            super(itemView);
            this.songTitle = itemView.findViewById(R.id.songTitle);
            this.songBeginning = itemView.findViewById(R.id.songBeginning);
            relativeLayout = itemView.findViewById(R.id.songItemRelativeLayout);
            bookPage = itemView.findViewById(R.id.book_page);
            icon = itemView.findViewById(R.id.songIcon);
            favStar = itemView.findViewById(R.id.fav_star);
        }
    }


    public interface OnClickListener {
        void onClick(Song selectedSong, View view);
        //void onClick2(Song song);
    }
}
