package de.thefeiter.liedgutverzeichnis.ui.main

import android.content.Context
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout
import de.thefeiter.liedgutverzeichnis.MainActivity.Companion.getAppDb
import de.thefeiter.liedgutverzeichnis.R
import de.thefeiter.liedgutverzeichnis.api.syncer.SyncerBookmarks

class FragmentBookmarks(private val mContext: Context) : Fragment() {
    private val db = getAppDb()
    private lateinit var recyclerView: RecyclerView

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        // Inflate the layout for this fragment
        val fragmentView = inflater.inflate(R.layout.fragment_bookmarks, container, false)
        recyclerView = fragmentView.findViewById(R.id.bookmarksRecyclerView)


        //load existing books
        val bookmarkSongs = db.songDao().getAllBookmarks()


        //initialize book list adapter
        val adapter = AdapterSongList(bookmarkSongs)
        recyclerView.setHasFixedSize(true)
        recyclerView.layoutManager = LinearLayoutManager(mContext)
        recyclerView.adapter = adapter

        val swipeRefreshLayout =
            fragmentView.findViewById<SwipeRefreshLayout>(R.id.bookmarksRefreshLayout)
        swipeRefreshLayout.setOnRefreshListener {
            val bmSyncer = SyncerBookmarks(mContext, getAppDb(), {}) {
                Toast.makeText(mContext, R.string.sync_bookmark_complete, Toast.LENGTH_LONG).show()
                swipeRefreshLayout.isRefreshing = false
                adapter.songs = db.songDao().getAllBookmarks()
                adapter.notifyDataSetChanged()
            }
            bmSyncer.sync()
        }


        return fragmentView
    }

    override fun onResume() {
        super.onResume()

        val adapter = recyclerView.adapter as AdapterSongList

        adapter.songs = db.songDao().getAllBookmarks()

        adapter.notifyDataSetChanged()
    }
}