package de.thefeiter.liedgutverzeichnis.ui.main

import android.content.Context
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout
import de.thefeiter.liedgutverzeichnis.MainActivity.Companion.getAppDb
import de.thefeiter.liedgutverzeichnis.R
import de.thefeiter.liedgutverzeichnis.SORT_BY_TITLE
import de.thefeiter.liedgutverzeichnis.api.syncer.SyncerFavorites

class FragmentFavorites(private val mContext: Context) : Fragment() {
    private val db = getAppDb()
    private lateinit var recyclerView: RecyclerView

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        // Inflate the layout for this fragment
        val fragmentView = inflater.inflate(R.layout.fragment_favorites, container, false)
        recyclerView = fragmentView.findViewById(R.id.favoritesRecyclerView)


        //load existing books
        val songs = db.songDao().getAllFavorites()


        //initialize book list adapter
        val adapter = AdapterSongList(songs)
        recyclerView.setHasFixedSize(true)
        recyclerView.layoutManager = LinearLayoutManager(mContext)
        recyclerView.adapter = adapter

        val swipeRefreshLayout =
            fragmentView.findViewById<SwipeRefreshLayout>(R.id.favoritesRefreshLayout)
        swipeRefreshLayout.setOnRefreshListener {
            val favSyncer = SyncerFavorites(
                mContext,
                getAppDb(),
                {}
            ) {
                Toast.makeText(mContext, R.string.sync_favorite_complete, Toast.LENGTH_LONG).show()
                swipeRefreshLayout.isRefreshing = false
                adapter.songs = db.songDao().getAllFavorites()
                adapter.notifyDataSetChanged()
            }
            favSyncer.sync()
        }
        return fragmentView
    }

    override fun onResume() {
        super.onResume()
        // Inflate the layout for this fragment
        val adapter = recyclerView.adapter as AdapterSongList?
        adapter!!.songs = db.songDao().getAllFavorites()
        adapter.sort(SORT_BY_TITLE)
        adapter.notifyDataSetChanged()
    }
}