package de.thefeiter.liedgutverzeichnis.ui.settings

import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.CheckBox
import android.widget.CompoundButton
import android.widget.RelativeLayout
import android.widget.TextView
import androidx.recyclerview.widget.RecyclerView
import de.thefeiter.liedgutverzeichnis.MainActivity.Companion.getAppDb
import de.thefeiter.liedgutverzeichnis.MainActivity.Companion.timestamp
import de.thefeiter.liedgutverzeichnis.R
import de.thefeiter.liedgutverzeichnis.objects.Book
import de.thefeiter.liedgutverzeichnis.objects.BookSetting
import de.thefeiter.liedgutverzeichnis.roomdb.AppDb

class ActiveBooksAdapter(private val books: List<Book>) :
    RecyclerView.Adapter<ActiveBooksAdapter.ViewHolder>() {
    private val db: AppDb = getAppDb()
    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
        val layoutInflater = LayoutInflater.from(parent.context)
        val listItem = layoutInflater.inflate(R.layout.item_settings, parent, false)
        return ViewHolder(listItem)
    }

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {


        // to prevent recycled viewHolder to change switch states
        holder.bookSwitch.setOnCheckedChangeListener(null)
        holder.bookName.text = books[position].name
        val book = books[position]
        val allBookSettings = db.bookSettingDao().getAll()
        val bookActive = allBookSettings.stream()
            .anyMatch { bs: BookSetting -> bs.active && book.id == bs.bookId }
        holder.bookSwitch.isChecked = bookActive

        //bei click on song
        holder.bookSwitch.setOnCheckedChangeListener { buttonView: CompoundButton?, isChecked: Boolean ->
            val timestamp = timestamp
            if (db.bookSettingDao().exists(book.id)) {
                db.bookSettingDao().update(isChecked, timestamp, book.id)
            } else {
                val bookSetting = BookSetting()
                bookSetting.active = isChecked
                bookSetting.bookId = book.id
                bookSetting.updated = timestamp
                db.bookSettingDao().insert(bookSetting)
            }
        }
        holder.relativeLayout.setOnClickListener { view: View? -> holder.bookSwitch.toggle() }
    }

    fun invertSelection() {
        val timestamp = timestamp
        books.forEach { book ->
            db.bookSettingDao().invertAll(timestamp)
            if (!db.bookSettingDao().exists(book.id)) {
                val bookSetting = BookSetting()
                bookSetting.active = true
                bookSetting.bookId = book.id
                bookSetting.updated = timestamp
                db.bookSettingDao().insert(bookSetting)
            }
            notifyItemChanged(books.indexOf(book))
        }
    }


    override fun getItemCount(): Int {
        return books.size
    }

    class ViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        val bookName: TextView = itemView.findViewById(R.id.settings_bookName)
        val bookSwitch: CheckBox = itemView.findViewById(R.id.bookActive_switch)
        val relativeLayout: RelativeLayout = itemView.findViewById(R.id.songItemRelativeLayout)

    }

}