package de.thefeiter.liedgutverzeichnis.ui.songdetails

import android.content.Context
import android.content.Intent
import android.view.View
import androidx.fragment.app.FragmentManager
import de.thefeiter.liedgutverzeichnis.CATEGORY_ID
import de.thefeiter.liedgutverzeichnis.R
import de.thefeiter.liedgutverzeichnis.SongChooser
import de.thefeiter.liedgutverzeichnis.objects.Category
import de.thefeiter.liedgutverzeichnis.objects.Song
import de.thefeiter.liedgutverzeichnis.roomdb.AppDb
import de.thefeiter.liedgutverzeichnis.ui.dialogs.CategorySuggestionDialog
import de.thefeiter.liedgutverzeichnis.ui.views.CategoryPill

class CategoryPillMaker(private val context: Context, private val db: AppDb) {

    val normalPillOnClick = { category: Category ->
        //start new activity with song object by intent
        val intent = Intent(context, SongChooser::class.java)
        intent.putExtra(CATEGORY_ID, category.id)
        context.startActivity(intent)
    }


    fun normalPill(category: Category): View {
        return normalPill(category, normalPillOnClick)
    }

    fun normalPill(category: Category, onClick: (Category) -> Unit): View {
        val categoryPill = CategoryPill(context, null)

        categoryPill.label.text = category.name
        categoryPill.setColor(category.color)

        categoryPill.setOnClickListener {
            onClick(category)
        }
        return categoryPill
    }


    fun newPill(song: Song, fragmentManager: FragmentManager): View {
        val categoryPill = CategoryPill(context, null)

        categoryPill.setNew()

        categoryPill.label.setText(R.string.pill_new)

        categoryPill.setOnClickListener {
            try {
                // try to reshow setup dialog
                val categorySuggestionDialog = CategorySuggestionDialog(db, song, context)
                categorySuggestionDialog.show(fragmentManager, null)
            } catch (e: ClassCastException) {
                e.printStackTrace()
            }
        }
        return categoryPill
    }


}