package de.thefeiter.liedgutverzeichnis.ui.topflop

import android.content.Intent
import android.os.Bundle
import android.view.MenuItem
import android.view.View
import android.widget.TextView
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import androidx.appcompat.widget.Toolbar
import com.android.volley.Response
import com.android.volley.toolbox.JsonObjectRequest
import de.thefeiter.liedgutverzeichnis.CATEGORY_ID
import de.thefeiter.liedgutverzeichnis.MainActivity
import de.thefeiter.liedgutverzeichnis.R
import de.thefeiter.liedgutverzeichnis.SongChooser
import de.thefeiter.liedgutverzeichnis.api.ApiTest
import de.thefeiter.liedgutverzeichnis.api.MySingleton
import de.thefeiter.liedgutverzeichnis.api.loader.LoaderArray
import de.thefeiter.liedgutverzeichnis.helpers.SnackbarHelper
import de.thefeiter.liedgutverzeichnis.objects.Category
import de.thefeiter.liedgutverzeichnis.objects.Song
import de.thefeiter.liedgutverzeichnis.objects.User
import de.thefeiter.liedgutverzeichnis.ui.songdetails.SongDetails
import de.thefeiter.liedgutverzeichnis.ui.views.Button
import de.thefeiter.liedgutverzeichnis.ui.views.CategoryPill
import de.thefeiter.liedgutverzeichnis.ui.views.SongPill
import org.json.JSONArray
import org.json.JSONObject


private const val TOP = 1
private const val FLOP = 2
private const val SKIP = 3

const val CATEGORY = 10
const val FOLLOWING_SONGS = 11


class CatTopFlopActivity : TopFlopActivity(CATEGORY)
class SongTopFlopActivity : TopFlopActivity(FOLLOWING_SONGS)


open class TopFlopActivity(
    private val mode: Int
) : AppCompatActivity() {


    private lateinit var getAllEndpoint: CharSequence
    private lateinit var topFlopActionPrefix: CharSequence


    private val db = MainActivity.getAppDb()
    private var song: Song? = null
    private var followingSong: Song? = null
    private var category: Category? = null
    private var user = User()
    private var suggestionId: Int = 0

    private lateinit var buttonSkip: Button
    private lateinit var buttonTop: Button
    private lateinit var buttonFlop: Button

    private lateinit var songPill: SongPill
    private lateinit var followingSongPill: SongPill
    private lateinit var categoryPill: CategoryPill

    private lateinit var upperText: TextView
    private lateinit var lowerText: TextView

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_topflop)

        ApiTest(this).test(db, {
            onCreateChecked()
        }, {
            finish()
        })

    }

    private fun onCreateChecked() {

        //adding the back button functionality
        val myChildToolbar = findViewById<Toolbar>(R.id.topFlopToolbar)
        setSupportActionBar(myChildToolbar)

        // Get a support ActionBar corresponding to this toolbar and enable the Up button
        supportActionBar!!.setDisplayHomeAsUpEnabled(true)


        buttonSkip = findViewById(R.id.topflop_skip)
        buttonTop = findViewById(R.id.topflop_top)
        buttonFlop = findViewById(R.id.topflop_flop)

        songPill = findViewById(R.id.topflop_song_pill)
        followingSongPill = findViewById(R.id.topflop_following_song_pill)
        categoryPill = findViewById(R.id.topflop_category_pill)

        upperText = findViewById(R.id.topflop_text_upper)
        lowerText = findViewById(R.id.topflop_text_lower)


        buttonSkip.setOnClickListener { handleAction(SKIP) }
        buttonTop.setOnClickListener { handleAction(TOP) }
        buttonFlop.setOnClickListener { handleAction(FLOP) }


        when (mode) {
            CATEGORY -> {
                getAllEndpoint = getString(R.string.endpoint_topflop_category_get)
                topFlopActionPrefix = getString(R.string.endpoint_topflop_category_action_prefix)

                upperText.text = getString(R.string.topflop_category_upper_text)
                lowerText.text = getString(R.string.topflop_categroy_lower_text)
            }
            FOLLOWING_SONGS -> {
                getAllEndpoint = getString(R.string.endpoint_topflop_song_get)
                topFlopActionPrefix = getString(R.string.endpoint_topflop_song_action_prefix)

                upperText.text = getString(R.string.topflop_song_upper_text)
                lowerText.text = getString(R.string.topflop_song_lower_text)

            }
            else -> {
                finishWithErrorMessage()
            }
        }

        getSuggestion()

    }

    private fun processJsonSuggestion(suggestion: JSONObject) {

        song = db.songDao().findById(suggestion.getInt(getString(R.string.json_key_songId)))
        suggestionId = suggestion.getInt(getString(R.string.json_key_suggestionId))

        if (suggestion.has(getString(R.string.json_key_categoryId))) {
            category = db.categoryDao()
                .findById(suggestion.getInt(getString(R.string.json_key_categoryId)))
        }

        if (suggestion.has(getString(R.string.json_key_suggestedSongId))) {
            followingSong = db.songDao()
                .findById(suggestion.getInt(getString(R.string.json_key_suggestedSongId)))
        }

        updatePills()

    }

    private fun getSuggestion() {
        val loader = object : LoaderArray(
            this,
            db,
            {},
            {},
            getString(R.string.api_base_url) + getAllEndpoint
        ) {
            override suspend fun onResponse(response: JSONArray) {

                if (response.length() == 0) {
                    Toast.makeText(
                        ctx,
                        ctx.getString(R.string.topflop_no_suggestions),
                        Toast.LENGTH_SHORT
                    ).show()
                    finish()
                    return
                }
                val suggestion = response.get(0) as JSONObject
                processJsonSuggestion(suggestion)
            }
        }

        loader.load()

    }

    private fun handleAction(action: Int) {

        var endpointAction = ""
        when (action) {
            TOP -> {
                endpointAction = getString(R.string.endpoint_topflop_action_top)
            }
            FLOP -> {
                endpointAction = getString(R.string.endpoint_topflop_action_flop)
            }
            SKIP -> {
                endpointAction = getString(R.string.endpoint_topflop_action_skip)
            }
        }

        val authRequest: JsonObjectRequest = object : JsonObjectRequest(
            Method.GET,
            getString(R.string.api_base_url) + topFlopActionPrefix + suggestionId + "/" + endpointAction,
            null,
            Response.Listener {
                //on request success execute callback
                SnackbarHelper(findViewById(R.id.topflop_layout)).makeSnackbar(
                    getString(R.string.success), findViewById(
                        R.id.topflop_button_layout
                    )
                )
                getSuggestion()
            },
            Response.ErrorListener {
                //on error notify about error
                it.printStackTrace()
                finishWithErrorMessage()
            }) {
            /**
             * Passing some request headers
             */
            override fun getHeaders(): Map<String, String> {
                val headers = HashMap<String, String>()
                headers[getString(R.string.api_headers_token)] = user.apiToken
                return headers
            }
        }


        //if api responds, add auth check request
        MySingleton.getInstance(this).addToRequestQueue(authRequest)
    }

    fun updatePills() {
        updateSongPill()
        updateFollowingSongPill()
        updateCategoryPill()
    }


    private fun updateSongPill() {
        song?.let { song ->
            songPill.label.text = song.title
            songPill.setOnClickListener {
                //start new activity with song object by intent
                val intent = Intent(this, SongDetails::class.java)
                intent.putExtra(MainActivity.SONG_DETAILS, song.id)
                startActivity(intent)
            }
        }
    }


    private fun updateFollowingSongPill() {
        followingSong?.let { song ->
            followingSongPill.visibility = View.VISIBLE
            followingSongPill.label.text = song.title
            followingSongPill.setOnClickListener {
                //start new activity with song object by intent
                val intent = Intent(this, SongDetails::class.java)
                intent.putExtra(MainActivity.SONG_DETAILS, song.id)
                startActivity(intent)
            }
        }
    }


    private fun updateCategoryPill() {
        category?.let { category ->
            categoryPill.visibility = View.VISIBLE
            categoryPill.label.text = category.name
            categoryPill.setColor(category.color)
            categoryPill.setOnClickListener {
                //start new activity with song object by intent
                val intent = Intent(this, SongChooser::class.java)
                intent.putExtra(CATEGORY_ID, category.id)
                startActivity(intent)
            }
        }
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {

        val itemId = item.itemId
        if (itemId == android.R.id.home) {
            finish()
            return true
        }
        // If we got here, the user's action was not recognized.
        // Invoke the superclass to handle it.
        return super.onOptionsItemSelected(item)

    }

    fun finishWithErrorMessage() {
        Toast.makeText(
            this,
            getString(R.string.message_something_went_wrong),
            Toast.LENGTH_SHORT
        ).show()
        finish()
    }
}
