package de.thefeiter.liedgutverzeichnis.ui.views

import android.content.Context
import android.content.res.ColorStateList
import android.graphics.drawable.GradientDrawable
import android.graphics.drawable.RippleDrawable
import android.util.AttributeSet
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import de.thefeiter.liedgutverzeichnis.R


class Button(context: Context, attrs: AttributeSet?) : LinearLayout(context, attrs) {

    var label: TextView
    var icon: ImageView
    var layout: LinearLayout

    init {
        inflate(context, R.layout.button, this)
        label = findViewById(R.id.button_text)
        icon = findViewById(R.id.button_icon)
        layout = findViewById(R.id.button_layout)


        val attributes = context.obtainStyledAttributes(attrs, R.styleable.Button)

        for (i in 0 until attributes.indexCount) {
            when (val attr = attributes.getIndex(i)) {
                R.styleable.Button_text -> {
                    label.text = attributes.getString(attr)
                }
                R.styleable.Button_bgColor -> {
                    val color = attributes.getColorStateList(attr)
                    setBackground(color)
                }
                R.styleable.Button_icon -> {
                    val resId = attributes.getResourceId(attr, 0)
                    if (resId != 0) icon.setImageResource(resId)
                }
            }
        }
        attributes.recycle()

    }

    fun setImageResource(resId: Int) {
        icon.setImageResource(resId)
    }

    fun setText(text: String) {
        label.text = text
    }

    private fun setBackground(colorStateList: ColorStateList?) {
        val background = layout.background as RippleDrawable
        val bgShape =
            background.findDrawableByLayerId(R.id.button_background_base) as GradientDrawable
        bgShape.color = colorStateList
    }
}